/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qgeneticalgorithm.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//	QGeneticAlgorithm class
//
///////////////////////////////////////////////////////////

//---------------------------------------------------------
QGeneticAlgorithm::QGeneticAlgorithm (void) {
	_fout.open ("ga.log");
}

//---------------------------------------------------------
QGeneticAlgorithm::QGeneticAlgorithm (const QGeneticAlgorithm& argEA) : QEvolutionaryAlgorithm(argEA) {
}

//---------------------------------------------------------
QGeneticAlgorithm::~QGeneticAlgorithm (void) {
	_fout.close ();
}

//---------------------------------------------------------
QGeneticAlgorithm& QGeneticAlgorithm::operator= (const QGeneticAlgorithm& argEA) {
	return (*this);
}

//---------------------------------------------------------
int QGeneticAlgorithm::setParameters (const QEAParameters& argParams) {
	this->QEvolutionaryAlgorithm::setParameters (argParams);

	return Q_OK;
}

//---------------------------------------------------------
int QGeneticAlgorithm::run () {
	if (_params.generationsNumber > 0) {
		int i;

		_summary.reset ();
		_summary.startClock = clock ();

		initializeComponents ();

		_fout<<"\nStarting a new run:\n"
			<<"-------------------------------\n";

		for (i=0; i<_params.generationsNumber; i++) {

			_summary.popSizes[ _summary.generation ] = _popul->getSize();
			evaluatePopulation ();
			selectParentIndividuals ();
			selectElite ();
			crossPopulation ();
			mutatePopulation ();
			nextGeneration ();

			_summary.generation++;

			if (! (_summary.generation % 10)) {
				printf(".");
			}
		}

		_summary.finishClock = clock ();
		_summary.time = (double) (_summary.finishClock - _summary.startClock) / CLOCKS_PER_SEC;

		printf("\n");

		evaluatePopulationNoStats ();
		_popul->printPopulation (_fout);

		_fout<<"Run duration: "<<_summary.time<<" seconds\n";

		return Q_OK;
	} else {
		cerr<<"\n\n[QGeneticAlgorithm::run] error:\n"
			<<"\tGenerations number is undefined! (_params.generationsNumber = "<<_params.generationsNumber<<")\n\n";
		return Q_FAILURE;
	}
}

QEvolutionaryAlgorithm* QGeneticAlgorithm::clone () {
	return new QGeneticAlgorithm (*this);
}

//QEvolutionaryAlgorithm* QGeneticAlgorithm::clone () const {
//	return new QGeneticAlgorithm (*this);
//}

// int printResults (__VECTOR_RUN_SUMMARY& argS, ostream& out)
//	Prints algorithm specific results into the passed output stream
int QGeneticAlgorithm::printResults (__VECTOR_RUN_SUMMARY& argS, ostream& out) {
	return Q_OK;
}

// int printName (ostream& out)
//	Prints algorithm name into the specified output stream
int QGeneticAlgorithm::printName (ostream& out) {
	out<<"Genetic algorithm\n";
	return Q_OK;
}
