/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef	Q_SELFADAPTIVE_REGULATORY_NETWORK_2_H
#define	Q_SELFADAPTIVE_REGULATORY_NETWORK_2_H

#include "qevolutionaryalgorithm.h"

namespace ECWorkshop {

//////////////////////////////////////////////////////
//
//	Self-Adaptive Regulatory Network 2 class
//

class QSARN2 : public QEvolutionaryAlgorithm {
protected:

	QRNIndividual* _net;	// evolved [RN]
	QRNMutationBasic* _mut;	// [RN] mutation

	QSARN2 (const QSARN2& argEA);
	virtual QSARN2& operator= (const QSARN2& argEA);

public:
	QSARN2 (void);
	virtual ~QSARN2 (void);

	// int setParameters (const QGAParameters& argParams)
	//	Sets parameters of the EA and makes necessary initializations
	//	Returns Q_OK on success
	virtual int setParameters (const QEAParameters& argParams);

	// int run ()
	//	Performs the main routine of evolutionary algorithm/model
	//	Returns Q_OK on success
	virtual int run ();

	// QEvolutionaryAlgorithm* clone ()
	//	Makes copy of the existing object
	virtual QEvolutionaryAlgorithm* clone ();

	// int evaluatePopulation
	// Overloaded method
	virtual int evaluatePopulation ();

	// int evaluatePopulation
	// Overloaded method
	virtual int evaluatePopulationNoStats ();

	// int printResults (__VECTOR_RUN_SUMMARY& argS, ostream& out)
	//	Prints algorithm specific results into the passed output stream
	virtual int printResults (__VECTOR_RUN_SUMMARY& argS, ostream& out);

	// int printName (ostream& out)
	//	Prints algorithm name into the specified output stream
	virtual int printName (ostream& out);
};

};

#endif