/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_INDIVIDUAL_H
#define		Q_INDIVIDUAL_H

#include "../include/qec.h"
#include "../../qtools/qtools.h"
#include "qintegergene.h"

namespace ECWorkshop {

///////////////////////////////////////////////////////////
//
//		QIndividual class
//
///////////////////////////////////////////////////////////

class QIndividual {
protected:
	unsigned int _niche;	// niche id. If 0 then niche is unassigned
	QFitness _fitness;
	const QEAParameters* _params;	// pointer to the external EA parameters container

	virtual int createIndividual (const int argGenesNumber) = 0;

public:
	QIndividual (void);
	QIndividual (const QIndividual& argInd);
	virtual ~QIndividual(void);

	virtual QIndividual& operator= (const QIndividual& argInd);
	virtual int assign (const QIndividual* argInd);

	virtual QIndividual* clone () const = 0;
	virtual QEncodingType getEncodingType () const = 0;
	virtual int toVectorDouble (__VECTOR_DOUBLE& target) = 0;
	virtual int toVectorInt (__VECTOR_INT& target) = 0;
	virtual int toVectorDouble (__VECTOR_DOUBLE& target) const = 0;
	virtual int toVectorInt (__VECTOR_INT& target) const = 0;
	virtual int print2Stream (ostream& out) const = 0;

	virtual int getSize () const = 0;
	virtual int addGene (const QGene* argGene) = 0;
	virtual int deleteGene (const int argIndex) = 0;
	virtual QGene* getGene (const int argIndex) = 0;
	virtual const QGene* getGene (const int argIndex) const = 0;
	virtual int setGene (const int argIndex, const QGene* argGene) = 0;
	virtual int clearIndividual () = 0;
	virtual bool equalsTo (const QIndividual* argInd) const = 0;

	int setNicheID (const unsigned int argN);
	unsigned int getNicheID () const;

	virtual int setParameters (const QEAParameters* argParams);

	const QFitness& getFitness () const;
	double getFitnessValue () const;
	int setFitness (const QFitness& argFitness);
	int setFitness (const double argFitness);

};

typedef		vector<QIndividual*>	__VECTOR_INDIVIDUAL_PTR;
typedef		vector<const QIndividual*>	__VECTOR_CONST_INDIVIDUAL_PTR;

};	// namespace

#endif
