/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_POPULATION_H
#define		Q_POPULATION_H

#include "../include/qec.h"
#include "qindividual.h"

namespace ECWorkshop {

///////////////////////////////////////////////////////////
//
//		QPopulation class
//
///////////////////////////////////////////////////////////

class QPopulation {
private:

protected:

	__VECTOR_INDIVIDUAL_PTR _inds;
	const QEAParameters* _params;	// pointer to the external EA parameters container

	// int createPopulation (const int)
	// Creates a population of given size with individuals with empty chromosomes
	//	Returns Q_OK on success
	virtual int createPopulation (const int argSize) = 0;

public:
	QPopulation (void);
	QPopulation (const QPopulation& argPopul);
	virtual ~QPopulation (void);

	virtual QPopulation& operator= (const QPopulation& argPopul);

	virtual QEncodingType getEncodingType () const = 0;
	virtual QPopulation* clone () const = 0;
	// int addIndividual ()
	// Adds default individual to the end of population
	//	Returns new population size on success or Q_FAILURE otherwise
	virtual int addIndividual () = 0;


	virtual int assign (const QPopulation* argPopul);

	// int setParameters (const QEAParameters& argParams)
	//
	// Sets necessary parameters and creates population and individuals
	//	Returns Q_OK on success
	//
	//	!!! Note: Population initialization should be done elsewhere !!!
	//
	virtual int setParameters (const QEAParameters& argParams);

	int getSize () const;

	// int insertIndividual ()
	//	Inserts given individual at the prescribed position
	int insertIndividual (const int pos, const QIndividual* argInd);
	// int addIndividual (const QIndividual*)
	// Adds given individual to the end of population
	//	Returns new population size on success or Q_FAILURE otherwise
	int addIndividual (const QIndividual* argInd);
	int deleteIndividual (const int argInd);
	// int deleteIndividuals (const int argInd)
	// Deletes [argNumber] individuals starting from the [argStart]
	//  position or until end of population is reached
	int deleteIndividuals (const int argStart, const int argNumber);
	QIndividual* getIndividual (const int argIndex);
	const QIndividual* getIndividual (const int argIndex) const;
	double getIndividualFitnessValue (const int argIndex) const;
	// QIndividual* getMinFitnessIndividual (const double argBound)
	// Returns [QIndividual] pointer to the individual with the
	//  least fitness that larger than argBound
	QIndividual* getMinFitnessIndividual (const double argBound);
	// int getMinFitnessIndividual (const double argBound) const
	// Returns index of the individual with the least fitness
	//  that larger than argBound
	int getMinFitnessIndividualIndex (const double argBound) const;
	int setIndividual (const int argIndex, const QIndividual* argInd);
	int clearPopulation ();

	void reserve (const int argSize);

	virtual int checkConvergence ();
	virtual int printPopulation (ostream& out) const;
};

typedef vector<QPopulation*>	__VECTOR_POPULATION_PTR;

};	// namespace

#endif
