/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qrnindividual.h"
#include "qrnpopulation.h"

using namespace ECWorkshop;

int QRNPopulation::createPopulation (const int argSize) {
	if ((int)_inds.size () > 0) {
		clearPopulation ();
	}

	int i;

	for (i=0; i<argSize; i++) {
		_inds.push_back (new QRNIndividual());
	}

	return Q_OK;
}

QRNPopulation::QRNPopulation (void) {
}

QRNPopulation::QRNPopulation (const QRNPopulation& argPopul)
: QPopulation(argPopul) {
}

QRNPopulation::~QRNPopulation (void) {
}

QRNPopulation& QRNPopulation::operator= (const QRNPopulation& argPopul) {
	this->QPopulation::operator = (argPopul);
	return (*this);
}

QEncodingType QRNPopulation::getEncodingType () const {
	return QET_REGULATORY_NETWORK;
}

QPopulation* QRNPopulation::clone () const {
	return new QRNPopulation (*this);
}

int QRNPopulation::addIndividual () {
	QRNIndividual tempInd;
	tempInd.setParameters (_params);
	_inds.push_back (tempInd.clone ());
	return (int)_inds.size();
}

int QRNPopulation::printPopulation  (ostream& out)  const {
	this->QPopulation::printPopulation (out);

	int i, j, popSize = (int)_inds.size(), size;
	double mean, var;
	__VECTOR_INT tempV, tempD;

	out<<"\n-------------------------------------------\nPopulation specific information:\n";

	// print average number of genes
	tempV.resize (popSize);
	for (i=0; i<popSize; ++i) {
		tempV[i] = _inds[i]->getSize ();
	}
	qstats (tempV, mean, var);
	out<<"Average number of genes:\t"<<mean<<"\n";
	out<<"Deviation of the number of genes:\t"<<var<<"\n";

	// print cumulative distribution of degrees over the all population
	for (i=0; i<popSize; ++i) {
		((QRNIndividual*)_inds[i])->calculateDegrees (tempV);

		size = (int)tempV.size ();
		for (j=0; j<size; ++j) {
			while ((int)tempD.size() <= tempV[j]) {
				tempD.push_back (0);
			}
			tempD[ tempV[j] ]++;
		}
	}
	out<<"Distribution of nodes degrees over the whole population:\t";
	size = (int)tempD.size ();
	for (i=0; i<size; ++i) {
		out<<" "<<tempD[i];
	}
	out<<"\n";


	return Q_OK;
}
