/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qstringindividual.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QStringIndividual class
//
///////////////////////////////////////////////////////////

int QStringIndividual::createIndividual (const int argGenesNumber) {
	_genes.resize (argGenesNumber);
	return Q_OK;
}

QStringIndividual::QStringIndividual (void) {
}

QStringIndividual::QStringIndividual (const QStringIndividual& argInd)
: QIndividual(argInd) {
	_genes.assign (argInd._genes.begin(), argInd._genes.end());
}

QStringIndividual::~QStringIndividual(void) {
	clearIndividual ();
}

QStringIndividual& QStringIndividual::operator= (const QStringIndividual& argInd) {
	this->QIndividual::operator = (argInd);
	_genes.assign (argInd._genes.begin(), argInd._genes.end());
	return (*this);
}

int QStringIndividual::assign (const QIndividual* argInd) {
	this->operator= ((QStringIndividual&)(*argInd));
	return Q_OK;
}

int QStringIndividual::getSize () const {
	return (int)_genes.size ();
}

int QStringIndividual::addGene (const QGene* argGene) {
	_genes.push_back ((QStringGene&)(*argGene));
	return (int)_genes.size ();
}

int QStringIndividual::deleteGene (const int argIndex) {
	if (argIndex < (int)_genes.size ()) {

		_genes.erase (_genes.begin() + argIndex);
		return (int)_genes.size ();

	} else {

		cerr<<"\n\n[QStringIndividual::deleteGene] error:\n"
			<<"\tIndex is out of genes array bounds\n\n";
		exit (0);

	}
}

QGene* QStringIndividual::getGene (const int argIndex) {
	if (argIndex < (int)_genes.size ()) {
		return &_genes[ argIndex ];
	} else {
		cerr<<"\n\n[QStringIndividual::getGene] error:\n"
			<<"\tIndex is out of genes array bounds\n\n";
		exit (0);
	}
}

const QGene* QStringIndividual::getGene (const int argIndex) const {
	if (argIndex < (int)_genes.size ()) {
		return &_genes[ argIndex ];
	} else {
		cerr<<"\n\n[QStringIndividual::getGene] error:\n"
			<<"\tIndex is out of genes array bounds\n\n";
		exit (0);
	}
}

int QStringIndividual::setGene (const int argIndex, const QGene* argGene) {
	if (argIndex < (int)_genes.size ()) {
		_genes[ argIndex ].assign (argGene);
		return Q_OK;
	} else {
		cerr<<"\n\n[QStringIndividual::setGene] error:\n"
			<<"\tIndex is out of genes array bounds (index = "<<argIndex<<")\n\n";
		exit (0);
	}
}

int QStringIndividual::clearIndividual () {
	_genes.clear ();
	return Q_OK;
}

bool QStringIndividual::equalsTo (const QIndividual* argInd) const {
	if (getEncodingType() == argInd->getEncodingType()) {
		const QStringIndividual* tempInd = (QStringIndividual*)argInd;
		if (_genes.size() == tempInd->_genes.size()) {
			__VECTOR_STRING_GENE::const_iterator iter = _genes.begin (), end = _genes.end();
			__VECTOR_STRING_GENE::const_iterator argIter = tempInd->_genes.begin ();

			for (; iter != end; iter++, argIter++) {
				if ((*iter).equalsTo (&(*argIter)) == false) {
					return false;
				}
			}
			return true;
		}
	}

	return false;
}

QIndividual* QStringIndividual::clone () const {
	return new QStringIndividual ((QStringIndividual&)(*this));
}

QEncodingType QStringIndividual::getEncodingType () const {
	return QET_STRING;
}

unsigned int QStringIndividual::getGeneValue (const int argIndex) const {
	return _genes[ argIndex ].getValue ();
}

int QStringIndividual::setGeneValue (const int argIndex, const unsigned int argValue) {
	if (argIndex < (int)_genes.size () && argIndex >= 0) {
		_genes[ argIndex ].setValue (argValue);
		return Q_OK;
	} else {
		cerr<<"\n\nQStringIndividual::setGeneValue error:\n"
			<<"\tIndex is out of genes array bounds\n\n";
		exit (0);
	}
}

int QStringIndividual::print2Stream (ostream& out) const {
	__VECTOR_STRING_GENE::const_iterator iter = _genes.begin (), end = _genes.end ();
	const QStringGene *tempGene = NULL;

	out<<"("<<_fitness.getValue()<<")\t";
	for (; iter != end; iter++) {
		tempGene = &(*iter);
		out<<tempGene->getValue()<<" ";
	}

	return Q_OK;
}

int QStringIndividual::toVectorDouble (__VECTOR_DOUBLE& target) {
	register int size = (int)_genes.size();
	target.resize (size);
	for (int i=0; i<size; i++) {
		target[i] = (double)_genes[i].getValue();
	}
	return Q_OK;
}

int QStringIndividual::toVectorInt (__VECTOR_INT& target) {
	int size = (int)_genes.size();
	target.resize (size);
	for (int i=0; i<size; i++) {
		target[i] = _genes[i].getValue();
	}
	return Q_OK;
}

int QStringIndividual::toVectorDouble (__VECTOR_DOUBLE& target) const {
	register int size = (int)_genes.size();
	target.resize (size);
	for (int i=0; i<size; i++) {
		target[i] = (double)_genes[i].getValue();
	}
	return Q_OK;
}

int QStringIndividual::toVectorInt (__VECTOR_INT& target) const {
	int size = (int)_genes.size();
	target.resize (size);
	for (int i=0; i<size; i++) {
		target[i] = _genes[i].getValue();
	}
	return Q_OK;
}
