/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_CROSSOVER_OPERATOR
#define		Q_CROSSOVER_OPERATOR

#include "../qoperator.h"

#include "../../include/qec.h"
#include "../../encodings/qpopulation.h"
#include "../parentselector/qparentselector.h"

#include <fstream>

namespace ECWorkshop {

///////////////////////////////////////////////////////////
//
//		QCrossoverOperator class
//
///////////////////////////////////////////////////////////

class QCrossoverOperator : public QOperator {
protected:
	const QParentSelector *_pSelector;
	QPopulation *_children;

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children) = 0;

	QCrossoverOperator (void);

public:
	QCrossoverOperator (const QEvolutionaryAlgorithm* owner);
	QCrossoverOperator (const QCrossoverOperator& argX);
	virtual ~QCrossoverOperator (void);

	virtual QCrossoverOperator& operator= (const QCrossoverOperator& argX);

	// int setParentSelector (const QParentSelector* argSelector)
	//
	// !!!
	// Should be called *before* [setPopulations] method
	// !!!
	virtual int setParentSelector (const QParentSelector* argSelector);
	virtual int setPopulations (const QPopulation* argParents, QPopulation* argChildren);

	// int operate ()
	// Performs crossover operation for parents that are selected
	// by [_pSelector] operator
	// Returns Q_OK on success
	virtual int operate ();

	virtual QEncodingType getEncodingType () const = 0;
	virtual QCrossoverOperator* clone () const = 0;
};

typedef vector<QCrossoverOperator*>	__VECTOR_CROSSOVER_PTR;

};

#endif
