/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_REALCODE_CROSSOVER_OPERATOR
#define		Q_REALCODE_CROSSOVER_OPERATOR

#include "qcrossover.h"

namespace ECWorkshop {

///////////////////////////////////////////////////////////
//
//		QRCCrossover class
//
// Notes:
// --------------------------
// Base crossover class for [real-coded] encoding
//
///////////////////////////////////////////////////////////

class QRCCrossover : public QCrossoverOperator {
protected:
	double _alpha;	// numerical parameter

	QRCCrossover (void);

public:

	QRCCrossover (const QEvolutionaryAlgorithm* owner);
	QRCCrossover (const QRCCrossover& argX);
	virtual ~QRCCrossover (void);

	virtual QRCCrossover& operator= (const QRCCrossover& argX);

	virtual QEncodingType getEncodingType () const;

	virtual int setParameters (const QEAParameters& argParams);
};


///////////////////////////////////////////////////////////
//
//		QRCCrossoverArithmetic class
//
// Notes:
// --------------------------
// Arithmetic crossover operator for [real-coded] encoding
//
///////////////////////////////////////////////////////////

class QRCCrossoverArithmetic : public QRCCrossover {
protected:

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* argChildren);

	QRCCrossoverArithmetic (void);

public:
	QRCCrossoverArithmetic (const QEvolutionaryAlgorithm* owner);
	QRCCrossoverArithmetic (const QRCCrossoverArithmetic& argX);
	virtual ~QRCCrossoverArithmetic (void);

	virtual QRCCrossoverArithmetic& operator= (const QRCCrossoverArithmetic& argX);

	virtual QCrossoverOperator* clone () const;
};


///////////////////////////////////////////////////////////
//
//		QRCCrossoverSBX class
//
// Notes:
// --------------------------
// SBX crossover operator for [real-coded] encoding
//
///////////////////////////////////////////////////////////

class QRCCrossoverSBX : public QRCCrossover {
protected:

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* argChildren);

	QRCCrossoverSBX (void);

public:
	QRCCrossoverSBX (const QEvolutionaryAlgorithm* owner);
	QRCCrossoverSBX (const QRCCrossoverSBX& argX);
	virtual ~QRCCrossoverSBX (void);

	virtual QRCCrossoverSBX& operator= (const QRCCrossoverSBX& argX);

	virtual QCrossoverOperator* clone () const;
};


///////////////////////////////////////////////////////////
//
//		QRCCrossoverBLX class
//
// Notes:
// --------------------------
// BLX crossover operator for [real-coded] encoding
//
///////////////////////////////////////////////////////////

class QRCCrossoverBLX : public QRCCrossover {
protected:

	virtual int cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* argChildren);

	QRCCrossoverBLX (void);

public:
	QRCCrossoverBLX (const QEvolutionaryAlgorithm* owner);
	QRCCrossoverBLX (const QRCCrossoverBLX& argX);
	virtual ~QRCCrossoverBLX (void);

	virtual QRCCrossoverBLX& operator= (const QRCCrossoverBLX& argX);

	virtual QCrossoverOperator* clone () const;
};

};	// namespace

#endif




