/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qstringcrossover.h"

#include "../../encodings/include.h"

#include <algorithm>

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QStringCrossover class
//
///////////////////////////////////////////////////////////

QStringCrossover::QStringCrossover (void) {
}

QStringCrossover::QStringCrossover (const QEvolutionaryAlgorithm* owner)
: QCrossoverOperator(owner) {
}

QStringCrossover::QStringCrossover (const QStringCrossover& argX)
: QCrossoverOperator(argX) {
}

QStringCrossover::~QStringCrossover (void) {
}

QStringCrossover& QStringCrossover::operator= (const QStringCrossover& argX) {
	this->QCrossoverOperator::operator = (argX);

	return (*this);
}

QEncodingType QStringCrossover::getEncodingType () const {
	return QET_STRING;
}


///////////////////////////////////////////////////////////
//
//		QStringCrossover1 class
//
// Notes:
// --------------------------
// 2-point crossover operator for [string] encoding
//
///////////////////////////////////////////////////////////

QStringCrossover1::QStringCrossover1 (void) {
}

QStringCrossover1::QStringCrossover1 (const QEvolutionaryAlgorithm* owner)
: QStringCrossover(owner) {
}

QStringCrossover1::QStringCrossover1 (const QStringCrossover1& argX)
: QStringCrossover(argX) {
}

QStringCrossover1::~QStringCrossover1 (void) {
}

QStringCrossover1& QStringCrossover1::operator= (const QStringCrossover1& argX) {
	this->QStringCrossover::operator = (argX);

	return (*this);
}

int QStringCrossover1::cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children) {

	int genesNumber = _params->chromosomeLength;
	int coPoint1;	// break points (somewhere inside the chromosome)
	int i;

	// select crossover points
	coPoint1 = (int)(QRandN() * (genesNumber-1) + 1);

	for (i=0; i<coPoint1; i++) {
		(*children)[0]->setGene (i, (*argParents)[0]->getGene( i ));
		(*children)[1]->setGene (i, (*argParents)[1]->getGene( i ));
	}

	for (i=coPoint1; i<genesNumber; i++) {
		(*children)[0]->setGene (i, (*argParents)[1]->getGene( i ));
		(*children)[1]->setGene (i, (*argParents)[0]->getGene( i ));
	}

	return Q_OK;
}

QCrossoverOperator* QStringCrossover1::clone () const {
	return new QStringCrossover1 ((QStringCrossover1&)(*this));
}


///////////////////////////////////////////////////////////
//
//		QStringCrossover2 class
//
// Notes:
// --------------------------
// 2-point crossover operator for [string] encoding
//
///////////////////////////////////////////////////////////

QStringCrossover2::QStringCrossover2 (void) {
}

QStringCrossover2::QStringCrossover2 (const QEvolutionaryAlgorithm* owner)
: QStringCrossover(owner) {
}

QStringCrossover2::QStringCrossover2 (const QStringCrossover2& argX)
: QStringCrossover(argX) {
}

QStringCrossover2::~QStringCrossover2 (void) {
}

QStringCrossover2& QStringCrossover2::operator= (const QStringCrossover2& argX) {
	this->QStringCrossover::operator = (argX);

	return (*this);
}

int QStringCrossover2::cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children) {
	int genesNumber = _params->chromosomeLength;
	int coPoint1, coPoint2;	// break points (somewhere inside the chromosome)
	int i;

	// select crossover points
	coPoint1 = (int)(QRandN() * (genesNumber));
	do {
		coPoint2 = (int)(QRandN() * (genesNumber));
	}	while (coPoint2 == coPoint1);

	if (coPoint1 > coPoint2) {
		swap (coPoint1, coPoint2);
	}

	for (i=0; i<coPoint1; i++) {
		(*children)[0]->setGene (i, (*argParents)[0]->getGene( i ));
		(*children)[1]->setGene (i, (*argParents)[1]->getGene( i ));
	}

	for (i=coPoint1; i<coPoint2; i++) {
		(*children)[0]->setGene (i, (*argParents)[1]->getGene( i ));
		(*children)[1]->setGene (i, (*argParents)[0]->getGene( i ));
	}

	for (i=coPoint2; i<genesNumber; i++) {
		(*children)[0]->setGene (i, (*argParents)[0]->getGene( i ));
		(*children)[1]->setGene (i, (*argParents)[1]->getGene( i ));
	}

	return Q_OK;
}

QCrossoverOperator* QStringCrossover2::clone () const {
	return new QStringCrossover2 ((QStringCrossover2&)(*this));
}


///////////////////////////////////////////////////////////
//
//		QStringCrossoverUniform class
//
// Notes:
// --------------------------
// Uniform crossover operator for [string] encoding
//
///////////////////////////////////////////////////////////

QStringCrossoverUniform::QStringCrossoverUniform (void) {
}

QStringCrossoverUniform::QStringCrossoverUniform (const QEvolutionaryAlgorithm* owner)
: QStringCrossover(owner) {
}

QStringCrossoverUniform::QStringCrossoverUniform (const QStringCrossoverUniform& argX)
: QStringCrossover(argX) {
}

QStringCrossoverUniform::~QStringCrossoverUniform (void) {
}

QStringCrossoverUniform& QStringCrossoverUniform::operator= (const QStringCrossoverUniform& argX) {
	this->QStringCrossover::operator = (argX);

	return (*this);
}

int QStringCrossoverUniform::cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children) {
	int genesNumber = _params->chromosomeLength;
	int i;

	for (i=0; i<genesNumber; i++) {
		if (QRandBool()) {
			(*children)[0]->setGene (i, (*argParents)[0]->getGene( i ));
			(*children)[1]->setGene (i, (*argParents)[1]->getGene( i ));
		} else {
			(*children)[1]->setGene (i, (*argParents)[0]->getGene( i ));
			(*children)[0]->setGene (i, (*argParents)[1]->getGene( i ));
		}
	}

	return Q_OK;
}

QCrossoverOperator* QStringCrossoverUniform::clone () const {
	return new QStringCrossoverUniform ((QStringCrossoverUniform&)(*this));
}


///////////////////////////////////////////////////////////
//
//		QStringCrossoverOrder class
//
// Notes:
// --------------------------
// Order crossover operator for [string] encoding. Should be
//  used for permutation encodings
//
///////////////////////////////////////////////////////////

QStringCrossoverOrder::QStringCrossoverOrder (void) {
}

QStringCrossoverOrder::QStringCrossoverOrder (const QEvolutionaryAlgorithm* owner)
: QStringCrossover(owner) {
}

QStringCrossoverOrder::QStringCrossoverOrder (const QStringCrossoverOrder& argX)
: QStringCrossover(argX) {
}

QStringCrossoverOrder::~QStringCrossoverOrder (void) {
}

QStringCrossoverOrder& QStringCrossoverOrder::operator= (const QStringCrossoverOrder& argX) {
	this->QStringCrossover::operator = (argX);

	return (*this);
}

int QStringCrossoverOrder::cross (const __VECTOR_CONST_INDIVIDUAL_PTR* argParents, __VECTOR_INDIVIDUAL_PTR* children) {
	int genesNumber = _params->chromosomeLength;
	int coPoint;
	int i, j, childrenCount = (int)children->size(), count;
	const QStringIndividual *parent1 = (QStringIndividual*)((*argParents)[0]),
						*parent2 = (QStringIndividual*)((*argParents)[1]);
	QStringIndividual *child;
	__VECTOR_UINT genes;

	for (j=0; j<childrenCount; j++) {
		coPoint = (int)(QRandN() * genesNumber);
		child = (QStringIndividual*)((*children)[j]);
		genes.clear ();
		genes.reserve (genesNumber);

		// copy first [coPoint] genes from the 1st parent
		for (i=0; i<coPoint; i++) {
			//(*children)[j]->setGene (i, (*argParents)[0]->getGene( i ));
			genes.push_back (parent1->getGeneValue( i ));
		}
//		cout<<"xover point = "<<coPoint<<"\n";
		// copy the other genes from the 2nd parent only if they weren't copied yet
		for (i=0, count=0; i<genesNumber && (coPoint+count)<genesNumber; i++) {
			if (find (genes.begin (), genes.end(), parent2->getGeneValue(i)) == genes.end()) {
				genes.push_back (parent2->getGeneValue(i));
				count++;
			}
			//if (! hasGene ((*children)[j], (*argParents)[1]->getGene( i ), coPoint+count)) {
			//	(*children)[j]->setGene (coPoint+count, (*argParents)[1]->getGene( i ));
			//	count++;
			//}
		}
//		printf ("coPoint: %i,\tcount: %i,\t (coPoint+count): %i\n", coPoint, count, (coPoint+count));


		if (_params->order != QO_UNKNOWN) {
			// if encoding ordering is used
			if (_params->order == QO_ASCENDING) {
				sort (genes.begin(), genes.end ());
			} else {
				sort (genes.begin (), genes.end (), greater<unsigned int>());
			}
		}

		if (genes.size() != genesNumber) {
			printf ("eh? %i\n", genes.size());
		}

		// copy genes value from temporary array into the children chromosome
		for (i=0; i<genesNumber; i++) {
			child->setGeneValue (i, genes[i]);
		}
	}


	return Q_OK;
}

QCrossoverOperator* QStringCrossoverOrder::clone () const {
	return new QStringCrossoverOrder ((QStringCrossoverOrder&)(*this));
}

bool QStringCrossoverOrder::hasGene (const QIndividual* argInd, const QGene* argGene, const int bound) {
	for (int i=0; i<bound; i++) {
		if (argInd->getGene(i)->equalsTo(argGene)) {
			return true;
		}
	}
	return false;
}
