/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qintegerinitializer.h"
#include "../../encodings/include.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QIntegerInitializer class
//
///////////////////////////////////////////////////////////


int QIntegerInitializer::useRandom () {
	int i, j, k;
	int popSize = _popul->getSize (), chrSize;
	int temp;
	QIntegerIndividual *tempInd = NULL;
	QIntegerGene *tempGene = NULL;

	for (i=0; i<popSize; i++) {

		tempInd = ( QIntegerIndividual* )_popul->getIndividual(i);
		chrSize = tempInd->getSize ();

		for (j=0; j<chrSize; j++) {
			tempGene = ( QIntegerGene* )tempInd->getGene(j);
			temp = 0;
			for (k=0; k<_params->geneSize; k++) {
				if (_unitsRatio > QRandN()) {
					temp |= bitMask[k];
				}
			}
//			temp = QRandN() * ((double)bitMask[ _params->geneSize ]);
			tempGene->setValue ((int)temp);
		}

	}	//	for (i=0; i<popSize; i++)

	return Q_OK;
}

int QIntegerInitializer::useUnmixed () {
	return useRandom ();
}

QIntegerInitializer::QIntegerInitializer (void) : _unitsRatio(0.5) {
}

QIntegerInitializer::QIntegerInitializer (const QEvolutionaryAlgorithm* owner)
: QInitializer(owner), _unitsRatio(0.5) {
}

QIntegerInitializer::QIntegerInitializer (const QIntegerInitializer& argInit)
: QInitializer(argInit), _unitsRatio(argInit._unitsRatio) {
}

QIntegerInitializer::~QIntegerInitializer (void) {
}

int QIntegerInitializer::operate () {
//	assert (_popul != NULL);

	if (_popul->getEncodingType() == QET_INTEGER) {

		switch (_params->initializationType) {
			case QIT_RANDOM:
				useRandom ();
				break;
			default:
				cerr<<"\n\n[QIntegerInitializer::operate] error:\n"
					<<"\tUnknown initialization type encountered! Using random initialization!\n"
					<<"(initialization type ID = "<<_params->initializationType<<")\n\n";
				useRandom ();
				return Q_FAILURE;
		}

		return Q_OK;

	} else {

		cerr<<"\n\n[QIntegerInitializer::operate] error:\n"
			<<"\tEncoding mismatch! (population encoding ID = "<<_popul->getEncodingType()<<")\n\n";
		exit (0);

	}	// if (_popul->getEncodingType() == QET_INTEGER)
}

QInitializer* QIntegerInitializer::clone () const {
	return (new QIntegerInitializer (*this));
}

QEncodingType QIntegerInitializer::getEncoding () const {
	return QET_INTEGER;
}

int QIntegerInitializer::setParameters (const QEAParameters& argParams) {
	this->QInitializer::setParameters (argParams);
	_unitsRatio = _params->unitsRatio;

	return Q_OK;
}
