/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qrcinitializer.h"
#include "../../encodings/include.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QRCInitializer class
//
///////////////////////////////////////////////////////////


int QRCInitializer::useRandom () {
	int i, j;
	int popSize = _popul->getSize (), chrSize;
	double temp;
	QRCIndividual *tempInd = NULL;
	QRCGene *tempGene = NULL;

	for (i=0; i<popSize; i++) {

		tempInd = ( QRCIndividual* )_popul->getIndividual(i);
		chrSize = tempInd->getSize ();

		for (j=0; j<chrSize; j++) {
			tempGene = ( QRCGene* )tempInd->getGene(j);
			temp = 10.0 + QRandGauss();
			tempGene->setValue ((double)temp);
		}

	}	//	for (i=0; i<popSize; i++)

	return Q_OK;
}

int QRCInitializer::useUnmixed () {
	// Not supported. Uses random initialization
	return useRandom ();
}

QRCInitializer::QRCInitializer (void) {
}

QRCInitializer::QRCInitializer (const QEvolutionaryAlgorithm* owner)
: QInitializer(owner) {
}

QRCInitializer::QRCInitializer (const QRCInitializer& argInit)
: QInitializer(argInit) {
}

QRCInitializer::~QRCInitializer (void) {
}

int QRCInitializer::operate () {
//	assert (_popul != NULL);

	if (_popul->getEncodingType() == QET_REAL) {

		switch (_params->initializationType) {
			case QIT_RANDOM:
				useRandom ();
				break;
			default:
				cerr<<"\n\n[QRCInitializer::operate] error:\n"
					<<"\tUnknown initialization type encountered! Using random initialization!\n"
					<<"(initialization type ID = "<<_params->initializationType<<")\n\n";
				useRandom ();
				return Q_FAILURE;
		}

		return Q_OK;

	} else {

		cerr<<"\n\n[QRCInitializer::operate] error:\n"
			<<"\tEncoding mismatch! (population encoding ID = "<<_popul->getEncodingType()<<")\n\n";
		exit (0);

	}	// if (_popul->getEncodingType() == QET_INTEGER)
}

QInitializer* QRCInitializer::clone () const {
	return (new QRCInitializer (*this));
}

QEncodingType QRCInitializer::getEncoding () const {
	return QET_REAL;
}

