/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qstringinitializer.h"
#include "../../encodings/include.h"

#include <algorithm>

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QStringInitializer class
//
///////////////////////////////////////////////////////////

int QStringInitializer::useRandom () {
	if (_params->arity > 0) {
		int i, j;
		int popSize = _popul->getSize (), chrSize;
		unsigned int temp;
		QStringIndividual *tempInd = NULL;
		QStringGene *tempGene = NULL;
		__VECTOR_UINT genes;

		for (i=0; i<popSize; i++) {

			tempInd = ( QStringIndividual* )_popul->getIndividual(i);
			chrSize = tempInd->getSize ();

			if (_params->permutation == false) {

				for (j=0; j<chrSize; j++) {
					tempGene = ( QStringGene* )tempInd->getGene(j);
					temp = (unsigned int)(QRandN() * (double)(_params->arity));
					// set random gene value
					tempGene->setValue (temp);
				}

			} else {	// permutation, sir!

				genes.clear ();
				genes.reserve (chrSize);
				for (j=0; j<chrSize; j++) {
					do {
						temp = (unsigned int)(QRandN() * (double)(_params->arity));
					} while (find (genes.begin(), genes.end(), temp) != genes.end());
					// set random gene value
					genes.push_back (temp);	// add new gene value to the temporary array
				}

				// sort genes
				if (_params->order == QO_ASCENDING) {
					sort (genes.begin (), genes.end ());
				} else if (_params->order == QO_DESCENDING) {
					sort (genes.begin (), genes.end (), greater<unsigned int>());
				}

				// copy contents of temp array into the chromosome
				for (j=0; j<chrSize; j++) {
					(( QStringGene* )tempInd->getGene(j))->setValue (genes[j]);
				}

			}


		}	//	for (i=0; i<popSize; i++)

		return Q_OK;

	} else {
		cerr<<"\n\n[QStringInitializer::operate] error:\n"
			<<"\tCan't create strings with non-positive arity (_params->arity = "<<_params->arity<<")\n\n";
		exit (0);
	}

	return Q_OK;
}

int QStringInitializer::useUnmixed () {
	if (_params->arity > 0) {
		int i, j, k;
		int size = (int)_popul->getSize(), genes;
		int part_size = size / _params->arity;
		QStringIndividual* tempInd;

		for (k=0; k<(_params->arity-1); k++) {
			for (i = k*part_size; i < (k+1)*part_size; i++) {
				tempInd = (QStringIndividual*)_popul->getIndividual(i);
				genes = tempInd->getSize();
				for (j=0; j<genes; j++) {
					tempInd->setGeneValue (j, k);
				}
			}
		}

		for (i=(_params->arity-1)*part_size; i<size; i++) {
			tempInd = (QStringIndividual*)_popul->getIndividual(i);
			genes = tempInd->getSize();
			for (j=0; j<genes; j++) {
				tempInd->setGeneValue (j, _params->arity-1);
			}
		}

		return Q_OK;

	} else {
		cerr<<"\n\n[QStringInitializer::operate] error:\n"
			<<"\tCan't create strings with non-positive arity (_params->arity = "<<_params->arity<<")\n\n";
		exit (0);
	}
}

QStringInitializer::QStringInitializer (void) {
}

QStringInitializer::QStringInitializer (const QEvolutionaryAlgorithm* owner)
: QInitializer (owner) {
}

QStringInitializer::QStringInitializer (const QStringInitializer& argInit)
: QInitializer(argInit) {
}

QStringInitializer::~QStringInitializer (void) {
}

int QStringInitializer::operate () {
//	assert (_popul != NULL);
	if (_popul->getEncodingType() == QET_STRING) {

		switch (_params->initializationType) {
			case QIT_RANDOM:
				useRandom ();
				break;
			case QIT_UNMIXED:
				useUnmixed ();
				break;
			default:
				cerr<<"\n\n[QStringInitializer::operate] error:\n"
					<<"\tUnknown initialization type encountered! Using random initialization!\n"
					<<"(initialization type ID = "<<_params->initializationType<<")\n\n";
				useRandom ();
				return Q_FAILURE;
		}

		return Q_OK;

	} else {
		cerr<<"\n\n[QStringInitializer::operate] error:\n"
			<<"\tEncoding mismatch! (population encoding ID = "<<_popul->getEncodingType()<<")\n\n";
		exit (0);
	}	//if (_popul->getEncodingType() == QET_STRING)
}

QInitializer* QStringInitializer::clone () const {
	return (new QStringInitializer (*this));
}

QEncodingType QStringInitializer::getEncoding () const {
	return QET_STRING;
}

