/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qintegermutation.h"

#include "../../encodings/include.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QIntegerMutation class
//
// Note:
// ------------
// Base class for mutation operators for [integer] encoding
//
///////////////////////////////////////////////////////////


QIntegerMutation::QIntegerMutation (void) {
}

QIntegerMutation::QIntegerMutation (const QEvolutionaryAlgorithm* owner)
: QMutationOperator(owner) {
}

QIntegerMutation::QIntegerMutation (const QIntegerMutation& argMut)
: QMutationOperator(argMut) {
}

QIntegerMutation::~QIntegerMutation (void) {
}

QIntegerMutation& QIntegerMutation::operator= (const QIntegerMutation& argMut) {
	this->QMutationOperator::operator = (argMut);
	return (*this);
}

QEncodingType QIntegerMutation::getEncodingType () const {
	return QET_INTEGER;
}


///////////////////////////////////////////////////////////
//
//		QIntegerMutationBitFlip class
//
// Note:
// ------------
// Simple bit-flip mutation operator for [integer] encoding
//
///////////////////////////////////////////////////////////

QIntegerMutationBitFlip::QIntegerMutationBitFlip (void) {
}

QIntegerMutationBitFlip::QIntegerMutationBitFlip (const QEvolutionaryAlgorithm* owner)
: QIntegerMutation(owner) {
}

QIntegerMutationBitFlip::QIntegerMutationBitFlip (const QIntegerMutationBitFlip& argMut)
: QIntegerMutation(argMut) {
}

QIntegerMutationBitFlip::~QIntegerMutationBitFlip (void) {
}

QIntegerMutationBitFlip& QIntegerMutationBitFlip::operator= (const QIntegerMutationBitFlip& argMut) {
	this->QIntegerMutation::operator= (argMut);
	return (*this);
}

int QIntegerMutationBitFlip::operate () {
//	assert (_mutant != NULL);

	int i, j, size = _params->chromosomeLength, geneSize = _params->geneSize;
	int geneValue;
	QIntegerIndividual *tempMut = (QIntegerIndividual*)_mutant;

	for (i=0; i<size; i++) {	// cycle through all genes
		geneValue = tempMut->getGeneValue (i);
		for (j=0; j<geneSize; j++) {	// cycle through all bits in current gene
			if (QRandN() < _params->mRate) {
				geneValue ^= (int)bitMask[j];
			}
		}
		tempMut->setGeneValue (i, geneValue);
	}

	return Q_OK;
}

QMutationOperator* QIntegerMutationBitFlip::clone () const {
	return new QIntegerMutationBitFlip ((QIntegerMutationBitFlip&)(*this));
}

