/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_INTEGER_MUTATION_OPERATOR_H
#define		Q_INTEGER_MUTATION_OPERATOR_H

#include "qmutation.h"

namespace ECWorkshop {

///////////////////////////////////////////////////////////
//
//		QIntegerMutation class
//
// Note:
// ------------
// Base class for mutation operators for [integer] encoding
//
///////////////////////////////////////////////////////////

class QIntegerMutation : public QMutationOperator {
protected:
	QIntegerMutation (void);

public:
	QIntegerMutation (const QEvolutionaryAlgorithm* owner);
	QIntegerMutation (const QIntegerMutation& argMut);
	virtual ~QIntegerMutation (void);

	virtual QIntegerMutation& operator= (const QIntegerMutation& argMut);

	virtual QEncodingType getEncodingType () const;
};

///////////////////////////////////////////////////////////
//
//		QIntegerMutationBitFlip class
//
// Note:
// ------------
// Simple bit-flip mutation operator for [integer] encoding
//
///////////////////////////////////////////////////////////

class QIntegerMutationBitFlip : public QIntegerMutation {
protected:
	QIntegerMutationBitFlip (void);

public:
	QIntegerMutationBitFlip (const QEvolutionaryAlgorithm* owner);
	QIntegerMutationBitFlip (const QIntegerMutationBitFlip& argMut);
	virtual ~QIntegerMutationBitFlip (void);

	virtual QIntegerMutationBitFlip& operator= (const QIntegerMutationBitFlip& argMut);

	virtual int operate ();
	virtual QMutationOperator* clone () const;
};

};	// namespace

#endif

