/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qrcmutation.h"

#include "../../encodings/include.h"

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QRCMutation class
//
// Note:
// ------------
// Base class for mutation operators for [real] encoding
//
///////////////////////////////////////////////////////////


QRCMutation::QRCMutation (void) {
}

QRCMutation::QRCMutation (const QEvolutionaryAlgorithm* owner)
: QMutationOperator(owner) {
}

QRCMutation::QRCMutation (const QRCMutation& argMut)
: QMutationOperator(argMut) {
}

QRCMutation::~QRCMutation (void) {
}

QRCMutation& QRCMutation::operator= (const QRCMutation& argMut) {
	this->QMutationOperator::operator = (argMut);
	return (*this);
}

QEncodingType QRCMutation::getEncodingType () const {
	return QET_REAL;
}


///////////////////////////////////////////////////////////
//
//		QRCMutationGauss class
//
// Note:
// ------------
// Gaussian mutation operator for [real] encoding
//
///////////////////////////////////////////////////////////

QRCMutationGauss::QRCMutationGauss (void) {
}

QRCMutationGauss::QRCMutationGauss (const QEvolutionaryAlgorithm* owner)
: QRCMutation(owner) {
}

QRCMutationGauss::QRCMutationGauss (const QRCMutationGauss& argMut)
: QRCMutation(argMut) {
}

QRCMutationGauss::~QRCMutationGauss (void) {
}

QRCMutationGauss& QRCMutationGauss::operator= (const QRCMutationGauss& argMut) {
	this->QRCMutation::operator= (argMut);
	return (*this);
}

int QRCMutationGauss::operate () {
	int i, size = _mutant->getSize();
	double geneValue;

	if (_mutant->getEncodingType() == QET_REAL) {
		QRCIndividual *tempMut = (QRCIndividual*)_mutant;

		for (i=0; i<size; i++) {	// cycle through all genes
			if (_params->mRate > QRandN()) {
				geneValue = tempMut->getGeneValue (i) + QRandGauss ();
				tempMut->setGeneValue (i, geneValue);
			}
		}

	} else if (_mutant->getEncodingType() == QET_INTEGER) {
		QIntegerIndividual *tempMut = (QIntegerIndividual*)_mutant;

		for (i=0; i<size; i++) {	// cycle through all genes
			if (_params->mRate > QRandN()) {
				// decode gene value and add random Gaussian variable
				geneValue = tempMut->getGeneValue (i)*_params->genePrecision + _params->geneOffset + QRandGauss ();
				// encode new gene value
				geneValue = (geneValue - _params->geneOffset) / _params->genePrecision;
				tempMut->setGeneValue (i, (int)geneValue);
			}
		}
	}

	return Q_OK;
}

QMutationOperator* QRCMutationGauss::clone () const {
	return new QRCMutationGauss ((QRCMutationGauss&)(*this));
}


