/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_REALCODE_MUTATION_OPERATOR_H
#define		Q_REALCODE_MUTATION_OPERATOR_H

#include "qmutation.h"

namespace ECWorkshop {

///////////////////////////////////////////////////////////
//
//		QRCMutation class
//
// Note:
// ------------
// Base class for mutation operators for [real] encoding
//
///////////////////////////////////////////////////////////

class QRCMutation : public QMutationOperator {
protected:
	QRCMutation (void);

public:
	QRCMutation (const QEvolutionaryAlgorithm* owner);
	QRCMutation (const QRCMutation& argMut);
	virtual ~QRCMutation (void);

	virtual QRCMutation& operator= (const QRCMutation& argMut);

	virtual QEncodingType getEncodingType () const;
};

///////////////////////////////////////////////////////////
//
//		QRCMutationGauss class
//
// Note:
// ------------
// Gaussian mutation operator for [real] encoding
//
///////////////////////////////////////////////////////////

class QRCMutationGauss : public QRCMutation {
protected:
	QRCMutationGauss (void);

public:
	QRCMutationGauss (const QEvolutionaryAlgorithm* owner);
	QRCMutationGauss (const QRCMutationGauss& argMut);
	virtual ~QRCMutationGauss (void);

	virtual QRCMutationGauss& operator= (const QRCMutationGauss& argMut);

	virtual int operate ();
	virtual QMutationOperator* clone () const;
};

};	// namespace

#endif


