/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_RN_MUTATION_OPERATOR_H
#define		Q_RN_MUTATION_OPERATOR_H

#include "qmutation.h"

namespace ECWorkshop {

// RN mutation constants
enum QRNMutationType {INPUT_WEIGHT_CHANGE = 0,
						OUTPUT_WEIGHT_CHANGE,
						NODE_BIAS_CHANGE,
						NODE_TYPE_CHANGE,
						INPUT_TYPE_CHANGE,
						OUTPUT_TYPE_CHANGE,
						TOGGLE_NODE_ON_OFF,
						ADD_DELETE_INPUT,
						ADD_DELETE_OUTPUT,
						SPLIT_NODE,
						ADD_DELETE_NODE,
						MUTATION_PARAMETERS};

// initial value for choice from
#define	INITIAL_VALUE	1


///////////////////////////////////////////////////////////
//
//		QRNMutation class
//
// Note:
// ------------
// Base class for mutation operators for [RN] encoding
//
///////////////////////////////////////////////////////////

class QRNMutation : public QMutationOperator {
protected:
	QRNMutation (void);

public:
	QRNMutation (const QEvolutionaryAlgorithm* owner);
	QRNMutation (const QRNMutation& argMut);
	virtual ~QRNMutation (void);

	virtual QRNMutation& operator= (const QRNMutation& argMut);

	virtual QEncodingType getEncodingType () const;
};

///////////////////////////////////////////////////////////
//
//		QRNMutationGauss class
//
// Note:
// ------------
// Gaussian mutation operator for [real] encoding
//
///////////////////////////////////////////////////////////

// TODO: Think how to add a new kind of signals in
//		[mutateChangeInput], [mutateChangeOutput] methods

class QRNMutationBasic : public QRNMutation {
protected:
	QRNMutationBasic (void);

public:

	int mutateAddDeleteNode (const double argMParam);
	int mutateAddDeleteInput (const double argMParam);
	int mutateAddDeleteOutput (const double argMParam);
	int mutateChangeNode (const double argMParam);
	int mutateChangeNodeBias (const double argMParam);
	int mutateChangeInput (const double argMParam);
	int mutateChangeOutput (const double argMParam);
	int mutateChangeInputWeight (const double argMParam);
	int mutateChangeOutputWeight (const double argMParam);
	int mutateOnOffNode (const double argMParam);
	int mutateSplitNode (const double argMParam);

public:
	QRNMutationBasic (const QEvolutionaryAlgorithm* owner);
	QRNMutationBasic (const QRNMutationBasic& argMut);
	virtual ~QRNMutationBasic (void);

	virtual QRNMutationBasic& operator= (const QRNMutationBasic& argMut);

	virtual int operate ();
	virtual QMutationOperator* clone () const;
};

};	// namespace

#endif


