/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qstringmutation.h"

#include "../../encodings/include.h"

#include <algorithm>

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QStringMutation class
//
// Note:
// ------------
// Base class for mutation operators for [string] encoding
//
///////////////////////////////////////////////////////////


QStringMutation::QStringMutation (void) {
}

QStringMutation::QStringMutation (const QEvolutionaryAlgorithm* owner)
: QMutationOperator(owner) {
}

QStringMutation::QStringMutation (const QStringMutation& argMut)
: QMutationOperator(argMut) {
}

QStringMutation::~QStringMutation (void) {
}

QStringMutation& QStringMutation::operator= (const QStringMutation& argMut) {
	this->QMutationOperator::operator = (argMut);
	return (*this);
}

QEncodingType QStringMutation::getEncodingType () const {
	return QET_STRING;
}


///////////////////////////////////////////////////////////
//
//		QStringMutationRandomFlip class
//
// Note:
// ------------
// Simple bit-flip mutation operator for integer encoding
//
///////////////////////////////////////////////////////////

QStringMutationRandomFlip::QStringMutationRandomFlip (void) {
}

QStringMutationRandomFlip::QStringMutationRandomFlip (const QEvolutionaryAlgorithm* owner)
: QStringMutation(owner) {
}

QStringMutationRandomFlip::QStringMutationRandomFlip (const QStringMutationRandomFlip& argMut)
: QStringMutation(argMut) {
}

QStringMutationRandomFlip::~QStringMutationRandomFlip (void) {
}

QStringMutationRandomFlip& QStringMutationRandomFlip::operator= (const QStringMutationRandomFlip& argMut) {
	this->QStringMutation::operator= (argMut);
	return (*this);
}

int QStringMutationRandomFlip::operate () {

	int i, k;
	unsigned newValue;
	QStringIndividual *tempMut = (QStringIndividual*)_mutant;
	int size = tempMut->getSize (), arity = _params->arity;
	__VECTOR_UINT genes;	// temporary array

//	printf ("size: %i\tarity:%i\n", size, arity);

	if (_params->permutation == true) {
		genes.resize (size);
		for (i=0; i<size; i++) {
			genes[i] = tempMut->getGeneValue(i);
		}
	}

	for (i=0; i<size; i++) {	// cycle through all genes
		if (QRandN() < _params->mRate) {
			if (_params->permutation == true) {

				// for permutation encoding
				// check is made if new gene value is already present
				do {
					newValue = (unsigned int)(QRandN() * arity);
				} while (find (genes.begin(), genes.end(), newValue) != genes.end());

				if (_params->order != QO_UNKNOWN) {
					// if ordered encoding is used
					// find position inside the chromosome where
					// the new gene value should be inserted
					if (newValue != genes[i]) {
						// delete old gene value
						genes.erase (genes.begin() + i);
						k = 0;
						if (_params->order == QO_ASCENDING) {
							while (genes[k] < newValue) {
								k++;
							}
						} else if (_params->order == QO_DESCENDING) {
							while (genes[k] > newValue) {
								k++;
							}
						}
						if (k < size) {
							// insert new gene value
							genes.insert (genes.begin() + k, newValue);
						} else {
							genes.push_back (newValue);
						}
					}
				} else {
					// no ordering is used and gene's current value is simply replaced by the new one
					genes[i] = newValue;
				}

			} else {

				// not permutation encoding
				tempMut->setGeneValue (i, (unsigned int)(QRandN() * arity));

			}	// if (_params->permutation == true)
		}
	}	// for (i=0; i<size; i++)

	// copy data from temporary array
	if (_params->permutation) {
		for (i=0; i<size; i++) {
			tempMut->setGeneValue(i, genes[i]);
		}
	}

	return Q_OK;
}

QMutationOperator* QStringMutationRandomFlip::clone () const {
	return new QStringMutationRandomFlip ((QStringMutationRandomFlip&)(*this));
}
