/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qcrowding.h"

#include "../../eas/include.h"
#include "../include.h"
#include "../../problems/include.h"

using namespace ECWorkshop;

/////////////////////////////////////////////////////////////
//
//	QCrowding class
//
/////////////////////////////////////////////////////////////

QCrowding::QCrowding () {
}

QCrowding::QCrowding (const QEvolutionaryAlgorithm* owner)
: QNichingOperator(owner) {
}

QCrowding::QCrowding (const QCrowding& argC)
: QNichingOperator (argC), _crowdingFactor(argC._crowdingFactor) {
}

QCrowding::~QCrowding () {
}

QCrowding& QCrowding::operator= (const QCrowding& argNiching) {
	this->QNichingOperator::operator = (argNiching);

	_crowdingFactor = argNiching._crowdingFactor;

	return (*this);
}

int QCrowding::setCrowdingFactor (const int argCF) {
	_crowdingFactor = argCF;
	return Q_OK;
}

int QCrowding::getCrowdingFactor () {
	return _crowdingFactor;
}

QNichingOperator* QCrowding::clone () {
	return new QCrowding (*this);
}

int QCrowding::operate () {
	// TODO
	return Q_OK;
}


/////////////////////////////////////////////////////////////
//
//	QDeterministicCrowding class
//
/////////////////////////////////////////////////////////////

QDeterministicCrowding::QDeterministicCrowding () {
}

QDeterministicCrowding::QDeterministicCrowding (const QEvolutionaryAlgorithm* owner)
: QNichingOperator(owner) {
}

QDeterministicCrowding::QDeterministicCrowding (const QDeterministicCrowding& argC)
: QNichingOperator (argC) {
}

QDeterministicCrowding::~QDeterministicCrowding () {
}

QDeterministicCrowding& QDeterministicCrowding::operator= (const QDeterministicCrowding& argNiching) {
	this->QNichingOperator::operator = (argNiching);

	return (*this);
}

int QDeterministicCrowding::setParents (QIndividual* argP1, QIndividual* argP2) {
	_parent1 = argP1;
	_parent2 = argP2;

	return Q_OK;
}

int QDeterministicCrowding::setChildren (QIndividual* argC1, QIndividual* argC2) {
	_child1 = argC1;
	_child2 = argC2;

	return Q_OK;
}

QNichingOperator* QDeterministicCrowding::clone () {
	return new QDeterministicCrowding (*this);
}

int QDeterministicCrowding::operate () {
	QDistance dist;
	double dist1, dist2;

	// mutate children
	_owner->getMutationOperator()->setMutant(_child1);
	_owner->getMutationOperator()->operate();
	_owner->getMutationOperator()->setMutant(_child2);
	_owner->getMutationOperator()->operate();

	// evaluate children
	_params->problem->evaluateIndividual (_child1);
	_params->problem->evaluateIndividual (_child2);

	dist1 = dist.measure(_parent1, _child1) + dist.measure(_parent2, _child2);
	dist2 = dist.measure(_parent1, _child2) + dist.measure(_parent2, _child1);

	if (dist1 < dist2) {
		// if the child perform better than its closest parent then replace the parent
		if (_child1->getFitnessValue () > _parent1->getFitnessValue()) {
			//printf ("\tchild1 performs worse than parent1! (%f > %f)\n", _child1->getFitnessValue (), _parent1->getFitnessValue());
			_child1->assign (_parent1);
		}
		if (_child2->getFitnessValue () > _parent2->getFitnessValue()) {
			//printf ("\tchild2 performs worse than parent2! (%f > %f)\n", _child2->getFitnessValue (), _parent2->getFitnessValue());
			_child2->assign (_parent2);
		}

	} else {

		// if the child perform better than its closest parent then replace the parent
		if (_child1->getFitnessValue () > _parent2->getFitnessValue()) {
			//printf ("\tchild1 performs worse than parent2! (%f > %f)\n", _child1->getFitnessValue (), _parent2->getFitnessValue());
			_child1->assign (_parent2);
		}
		if (_child2->getFitnessValue () > _parent1->getFitnessValue()) {
			//printf ("\tchild2 performs worse than parent1! (%f > %f)\n", _child2->getFitnessValue (), _parent1->getFitnessValue());
			_child2->assign (_parent1);
		}
	}	// if (dist1 < dist2)

	return Q_OK;
}
