/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qfitnesssharing.h"

#include "../../encodings/include.h"

#include <valarray>

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//	QFitnessSharing class
//
///////////////////////////////////////////////////////////

QFitnessSharing::QFitnessSharing () {
	_alpha = 1.0;
	_sigma = 1.0;
}

QFitnessSharing::QFitnessSharing (const QEvolutionaryAlgorithm* owner)
: QNichingOperator(owner) {
	_alpha = 1.0;
	_sigma = 1.0;
}

QFitnessSharing::QFitnessSharing (const QFitnessSharing& argNiching)
: QNichingOperator(argNiching), _alpha(argNiching._alpha),
_sigma(argNiching._sigma) {
}

QFitnessSharing::~QFitnessSharing () {
	_alpha = _sigma = 0.0;
}

QFitnessSharing& QFitnessSharing::operator= (const QFitnessSharing& argNiching) {
	this->QNichingOperator::operator = (argNiching);

	_alpha = argNiching._alpha;
	_sigma = argNiching._sigma;

	return (*this);
}

int QFitnessSharing::setAlpha (const double argAlpha) {
	_alpha = argAlpha;
	return Q_OK;
}
double QFitnessSharing::getAlpha () const {
	return _alpha;
}

int QFitnessSharing::setSigma (const double argSigma) {
	_sigma = argSigma;
	return Q_OK;
}
double QFitnessSharing::getSigma () const {
	return _sigma;
}

int QFitnessSharing::operate () {
	__TABLE_DOUBLE mdist;	// distances matrix
	unsigned int i, j;
	unsigned int size = _popul->getSize();
	QDistance dist;
	QIndividual* tempInd;

	//cout<<size<<"\n";
	//cout.flush ();

	// TODO: Simplyfied calculation of distances for submatrix problem
	//if (_popul->getEncodingType() == QET_INTEGER && _params->geneSize == 1) {
	//	// calculate units per each chromosome
	//	__VECTOR_INT units;

	//	units.assign (size, 0);
	//	for (i=0; i<size; i++) {
	//		for (j=0; j<_params->chromosomeLength; j++) {
	//			units[i] += _popul->getIndividual(i)->;
	//		}
	//	}
	//}

	// allocate temporal similarity matrix using _sigma as threshold distance
	//calculateDistanceMatrix (mdist);
	__VECTOR_INT units;
	unsigned int chrSize;

	units.resize (size);
	units.assign (size, 0);
	for (i=0; i<size; i++) {
		tempInd = _popul->getIndividual (i);
		chrSize = tempInd->getSize ();
		for (j=0; j<chrSize; j++) {
			units[i] += ((QIntegerIndividual*)tempInd)->getGeneValue(j);
		}
	}

	mdist.resize (size);
	//mdist.assign (size, 0);
	for (i=0; i<size; i++) {
		mdist[i].resize (size);
		mdist[i].assign (size, 0);
	}

	// calculate temporal similarity matrix using _sigma as threshold distance
	for (i=0; i<(size-1); i++) {
		tempInd = _popul->getIndividual(i);
		for (j=(i+1); j<size; j++) {
			//cout<<i<<" : "<<j<<"\n";
			//cout<<dist.measure (tempInd, _popul->getIndividual(j))<<"\n";
			//tempInd->print2Stream (cout);
			//cout<<"\n";
			//_popul->getIndividual(j)->print2Stream (cout);
			//cout<<"\n";
			//cout.flush ();
			if (abs(units[i] - units[j]) < _sigma) {
				mdist[j][i] = mdist[i][j] = dist.measure (tempInd, _popul->getIndividual(j));
			} else {
				mdist[j][i] = mdist[i][j] = _sigma + 1;
			}
			// mdist[j][i] = mdist[i][j] = dist.measure (tempInd, _popul->getIndividual(j));

			//cout<<i<<" : "<<j<<"\n";
			//cout<<temp<<"\n";
			//cout.flush ();
		}
	}

	// recalculate fitness;
	double sh = 1.0;	// sharing function value
	for (i=0; i<size; i++) {
		sh = 1.0;
		for (j=i+1; j<size; j++) {
			if (mdist[i][j] < _sigma) {
				sh += (1 - pow((mdist[i][j])/_sigma, _alpha));
			}	// else do nothing
		}
		_popul->getIndividual(i)->setFitness ( _popul->getIndividual(i)->getFitnessValue () / sh );
//		_popul->getIndividual(i)->setFitness ( _popul->getIndividual(i)->getFitnessValue () * sh );
	}

	return Q_OK;
}

QNichingOperator* QFitnessSharing::clone () {
	return new QFitnessSharing ((const QFitnessSharing&)(*this));
}

