/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_PARENT_SELECTOR_H
#define		Q_PARENT_SELECTOR_H

#include "../qoperator.h"

#include "../../encodings/qpopulation.h"

namespace ECWorkshop {

///////////////////////////////////////////////////////////
//
//		QParentSelector class
//
// Notes:
// --------------------------
// When some data within [QParentSelector] based class should be changed
// the 'mutable' keyword is to be used (see [QParentSelector::_parents]
// variable for example)
//
///////////////////////////////////////////////////////////

class QParentSelector : public QOperator {
private:
	// int operate ()
	// Is inaccessible from now onwards
	// NEVER call this function. It does nothing anyway
	//
	virtual int operate ();

protected:
	mutable const QPopulation* _parents;

	QParentSelector (void);

public:
	QParentSelector (const QEvolutionaryAlgorithm* owner);
	QParentSelector (const QParentSelector& argSelector);
	virtual ~QParentSelector (void);

	virtual QParentSelector& operator= (const QParentSelector& argSelector);

	virtual int setPopulation (const QPopulation* argParents) const;

	virtual const QIndividual* selectParent () const = 0;
	virtual QParentSelector* clone () const = 0;
};

typedef vector<QParentSelector*>	__VECTOR_PARENT_SELECTOR_PTR;

};	// namespace

#endif



