/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qsimplepopulationsizer.h"

#include <math.h>

using namespace ECWorkshop;


///////////////////////////////////////////////////////////
//
//		QSimplePopulationSizer class
//
///////////////////////////////////////////////////////////

QSimplePopulationSizer::QSimplePopulationSizer(void)
: _newPopSize(0), _direction(0), _errorOld(0), _rs(NULL), _minSize (10)  {
}

QSimplePopulationSizer::QSimplePopulationSizer(const QEvolutionaryAlgorithm* owner)
: QPopulationSizer(owner), _newPopSize(0), _direction(0),
_errorOld(0), _rs(NULL), _minSize (10)  {
}

QSimplePopulationSizer::QSimplePopulationSizer (const QSimplePopulationSizer& argSizer)
: QPopulationSizer(argSizer), _newPopSize(argSizer._newPopSize), _direction(argSizer._direction),
_errorOld(argSizer._errorOld), _rs(argSizer._rs), _minSize (argSizer._minSize) {
}

QSimplePopulationSizer::~QSimplePopulationSizer(void) {
}

QSimplePopulationSizer& QSimplePopulationSizer::operator= (const QSimplePopulationSizer& argSizer) {
	this->QPopulationSizer::operator = (argSizer);

	_newPopSize = argSizer._newPopSize;
	_direction = argSizer._direction;
	_errorOld = argSizer._errorOld;
	_rs = argSizer._rs;
	_minSize = argSizer._minSize;

	return (*this);
}

int QSimplePopulationSizer::setParameters (const QEAParameters& argParams) {
	int ret = this->QOperator::setParameters (argParams);

	reset ();
//	_newPopSize = _params->populationSize;
	return ret;
}

int QSimplePopulationSizer::setSummary (const QRunSummary& argRS) {
	_rs = &argRS;
	return Q_OK;
}

int QSimplePopulationSizer::reset () {
	_newPopSize = _params->populationSize;
	_direction = 0;
	_errorOld = 0.0;
	_minSize = (int)(2*((int)(1.443*log((double)_params->chromosomeLength * _params->geneSize)) + 2));//_params->chromosomeLength * _params->geneSize / 25 + 6;
	if (_minSize < 10) {
		_minSize = 10;
	}
	if (_newPopSize < _minSize) {
		_newPopSize = _minSize;
	}
	return Q_OK;
}

QPopulationSizer* QSimplePopulationSizer::clone () const {
	return new QSimplePopulationSizer (*this);
}

bool QSimplePopulationSizer::populationReady () const {
	//assert (_popul != NULL);
	//cout<<"QConstPopulationSizer::populationReady\n";
	//cout.flush ();
	return (_popul->getSize() < _newPopSize)? false : true; 
}

int QSimplePopulationSizer::operate () {
	//assert (_popul != NULL);
	//if (_rs->generation == 0) {
	//	_newPopSize = _params->populationSize;
	//	_fs.reset ();
	//}

	if (_popul->getSize() > _newPopSize) {
		_popul->deleteIndividuals (_newPopSize, _popul->getSize() - _newPopSize);
	}

	// define new population size
	int limit = (int)(1.443*log((double)_params->chromosomeLength * _params->geneSize));
	if (_rs->generation > limit) {
		if (_errorOld > _rs->bests[ _rs->generation ]) {
			// everything is OK, reduce population size
			_direction = -1;
		} else {
			// oops, we need more help
			_direction = 1;
		}
	} else {
		_direction = 1;
	}

	_newPopSize += _direction;// * _fs.getCurrent();
	if (_newPopSize < _minSize) {
		_newPopSize = _minSize;
	} else if (_newPopSize > _params->maxPopSize) {
		_newPopSize = _params->maxPopSize;
	}
	_errorOld = _rs->bests[ _rs->generation ];
	return Q_OK;
}

