/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#include "qsimplepopulationsizerfibonacci.h"

#include <math.h>

using namespace ECWorkshop;

///////////////////////////////////////////////////////////
//
//		QSimplePopulationSizerFibonacci class
//
///////////////////////////////////////////////////////////

QSimplePopulationSizerFibonacci::QSimplePopulationSizerFibonacci(void) {
	_fs.reset ();
}

QSimplePopulationSizerFibonacci::QSimplePopulationSizerFibonacci(const QEvolutionaryAlgorithm* owner)
: QSimplePopulationSizer(owner) {
	_fs.reset ();
}

QSimplePopulationSizerFibonacci::QSimplePopulationSizerFibonacci (const QSimplePopulationSizerFibonacci& argSizer)
: QSimplePopulationSizer(argSizer) {
	_fs = argSizer._fs;
//	_minSize = argSizer._minSize;
}

QSimplePopulationSizerFibonacci::~QSimplePopulationSizerFibonacci(void) {
}

QSimplePopulationSizerFibonacci& QSimplePopulationSizerFibonacci::operator= (const QSimplePopulationSizerFibonacci& argSizer) {
	this->QSimplePopulationSizer::operator = (argSizer);

	_fs = argSizer._fs;
//	_minSize = argSizer._minSize;

	return (*this);
}

int QSimplePopulationSizerFibonacci::reset () {
	this->QSimplePopulationSizer::reset ();
	_fs.reset ();
	//_minSize = (int)(2*((int)(1.443*log((double)_params->chromosomeLength * _params->geneSize)) + 2));//_params->chromosomeLength * _params->geneSize / 25 + 6;
	//if (_minSize < 10) {
	//	_minSize = 10;
	//}
	//if (_newPopSize < _minSize) {
	//	_newPopSize = _minSize;
	//}
	return Q_OK;
}

QPopulationSizer* QSimplePopulationSizerFibonacci::clone () const {
	return new QSimplePopulationSizerFibonacci ((QSimplePopulationSizerFibonacci&)(*this));
}

int QSimplePopulationSizerFibonacci::operate () {
	//MessageBox (NULL, "QSimplePopulationSizerFibonacci::operate", "QSimplePopulationSizerFibonacci::operate", MB_OK);
	//assert (_popul != NULL);
	//if (_rs->generation == 0) {
	//	_newPopSize = _params->populationSize;
	//	_fs.reset ();
	//}

	if (_popul->getSize() > _newPopSize) {
		_popul->deleteIndividuals (_newPopSize, _popul->getSize() - _newPopSize);
	}

	int limit = (int)(1.443*log((double)_params->chromosomeLength * _params->geneSize));
	if (_rs->generation > limit) {
		// define new population size
		if (_errorOld > _rs->bests[ _rs->generation ]) {
			// everything is OK, reduce population size
			if (_direction == 1) {
				_fs.reset ();
			}
			_direction = -1;
		} else {
			// oops, we need more help
			if (_direction == -1) {
				_fs.reset ();
			}
			_direction = 1;
		}
	} else {
		_direction = 1;
	}
	int step = _fs.getCurrent();
	if (3*step > _newPopSize) {	// step limitation
		step = _newPopSize/3;
//		_fs.reset ();
	}
	_newPopSize += (int)_direction * step;
	if (_newPopSize < _minSize) {
		_newPopSize = _minSize;
		_fs.reset ();
	} else if (_newPopSize > _params->maxPopSize) {
		_newPopSize = _params->maxPopSize;
		_fs.reset ();
	}
	_errorOld = _rs->bests[ _rs->generation ];
	++_fs;

	return Q_OK;
}
