/*****************************************

ECWorkshop -- evolutionary computation
toolkit library

Web: http://qai.narod.ru/ecw/
Mailto: qai@mail.ru

Copyright (C) 2007 by Yury Tsoy


This program is free software; you can
redistribute it and/or modify it under the
terms of the GNU General Public License as
published by the Free Software Foundation;
either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that
it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License
for more details.

You should have received a copy of the GNU
General Public License along with this
program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

*/

#ifndef		Q_EA_FACTORY_H
#define		Q_EA_FACTORY_H

#include "../include/qec.h"
#include "../encodings/include.h"
#include "initialization/qinitializer.h"
#include "crossover/qcrossover.h"
#include "mutation/qmutation.h"
#include "niching/qnichingoperator.h"
#include "parentselector/qparentselector.h"
#include "populationsizer/qpopulationsizer.h"
#include "selection/qselection.h"

namespace ECWorkshop {

class QEvolutionaryAlgorithm;

///////////////////////////////////////////////////////////
//
//		QEAFactory class
//
//	Each QEAFactory object has its owner ([_owner] variable)
// which determines the EA class posessing the factory
//
///////////////////////////////////////////////////////////

class QEAFactory {
private:
	QEAFactory (const QEAFactory& argFactory);
	virtual QEAFactory& operator= (const QEAFactory& argFactory);

protected:
	const QEvolutionaryAlgorithm* _owner;

	__VECTOR_POPULATION_PTR _populations;
	__VECTOR_INITIALIZER_PTR _inits;
	__VECTOR_NICHING_PTR _nichings;
	__VECTOR_SELECTION_PTR _selections;
	__VECTOR_PARENT_SELECTOR_PTR _pSelectors;
	__VECTOR_POPULATION_SIZER_PTR _sizers;
	__VECTOR_CROSSOVER_PTR _crossovers;
	__VECTOR_MUTATION_PTR _mutations;

	int deletePopulations ();
	int deleteInitializers ();
	int deleteNichings ();
	int deleteSelections ();
	int deleteParentSelectors ();
	int deletePopulationSizers ();
	int deleteCrossovers ();
	int deleteMutations ();

	QCrossoverOperator* createIntegerCrossover (const QCrossoverType& argXType);
	QMutationOperator* createIntegerMutation (const QMutationType& argMType);

	QCrossoverOperator* createStringCrossover (const QCrossoverType& argXType);
	QMutationOperator* createStringMutation (const QMutationType& argMType);

	QCrossoverOperator* createRCCrossover (const QCrossoverType& argXType);
	QMutationOperator* createRCMutation (const QMutationType& argMType);

	QMutationOperator* createRNMutation (const QMutationType& argMType);

	QEAFactory (void);

public:

	QEAFactory (const QEvolutionaryAlgorithm* argOwner);
	virtual ~QEAFactory (void);

	virtual QPopulation* createPopulation (const QEncodingType& argEncoding);
	virtual QInitializer* createInitializer (const QEncodingType& argEncoding);
	virtual QNichingOperator* createNichingOperator (const QNichingType& argNichingType);
	virtual QSelection* createSelection (const QSelectionType& argSelType);
	virtual QParentSelector* createParentSelector (const QParentSelectorType& argPSelType);
	virtual QPopulationSizer* createPopulationSizer (const QPopulationSizerType& argSizerType);
	virtual QCrossoverOperator* createCrossover (const QEncodingType& argEncoding, const QCrossoverType& argXType);
	virtual QMutationOperator* createMutation (const QEncodingType& argEncoding, const QMutationType& argMType);

	virtual int deleteComponents ();
	virtual int deleteLastCreatedPopulation ();
	virtual int deleteLastCreatedMutation ();
};

};	// namespace

#endif
