#include "qlightxml.h"
#include "qtools.h"

#include <fstream>
#include <stdio.h>

using namespace std;

QLightXMLIO::QLightXMLIO () {
}

QLightXMLIO::~QLightXMLIO () {
}

int QLightXMLIO::readTag (istream& in, QXMLTag& tag) {
	string str;
	bool close = false;
	int pos;

	getline (in, str, '<');
	// delete all CRs and trim [str]
	trim (str);

	if (!str.empty ()) {
		// trimmed [str] houses tag contents
//		cout<<"tag contents: "<<str<<"\n";
		tag._contents = str;
	}

	// look for embedded tags or current tag's end
	do {
		getline (in, str, '>');
		trim (str);
		pos = (int)str.find_first_of ('/');
		if (pos < 0) {	// if new tag is opened
			QXMLTag tempTag;

			// look for the new tag's name
			pos  = (int)str.find_first_of (' ');
			if (pos < 0) {	// [str] *is* a tag's name 
				tempTag._name = str;
			} else {
				tempTag._name = str.substr (0, pos);
			}
//			cout<<"embedded tag name: "<<tempTag._name<<"\n";
			tag._tags.push_back (tempTag);
			// read embedded tag
			readTag (in, tag._tags [ (int)tag._tags.size()-1 ]);

			// look for the tag contents
			getline (in, str, '<');
			trim (str);
			if (!str.empty()) {
				tag._contents += str;
			}
		} else {	// current tag is closed
			close = true;
		}
	} while (!close);

	return Q_OK;
}

int QLightXMLIO::writeTag (ostream& out, const QXMLTag& tag) const {
	out<<"<"<<tag._name<<">"<<tag._contents;
	if (!tag._tags.empty()) {	// if there are embedded tags
		QXMLTAG_LIST::const_iterator iter = tag._tags.begin (), end = tag._tags.end ();
		for (; iter != end; iter++) {
			out<<"\n";	// start each new tag from the new line
			writeTag (out, (*iter));
		}
		out<<"\n";
	}
	out<<"</"<<tag._name<<">";
	return Q_OK;

	//out<<"Tag info:\n------------------\n"
	//	<<"\tName:\t"<<tag._name<<"\n"
	//	<<"\tContents:\t"<<tag._contents<<"\n";
	//if (!tag._tags.empty()) {	// if there are embedded tags
	//	QXMLTAG_LIST::iterator iter = tag._tags.begin (), end = tag._tags.end ();
	//	int count=1;
	//	for (; iter != end; iter++, count++) {
	//		out<<"Embedded tag #"<<count<<":\n";
	//		writeTag (out, (*iter));
	//	}
	//}
	//return Q_OK;
}

int QLightXMLIO::appendTagToList (const QXMLTag& tag, const char* argName, QXMLTAG_LIST& argList) const {
	if (tag._name.compare (argName) == 0) {	// if names are equal
		argList.push_back (tag);
	}
	if (!tag._tags.empty()) {	// if there are embedded tags
		QXMLTAG_LIST::const_iterator iter = tag._tags.begin (), end = tag._tags.end ();
		for (; iter != end; iter++) {
			appendTagToList ((*iter), argName, argList);
		}
	}
	return Q_OK;
}


int QLightXMLIO::readFromFile (const char* argFilename) {
	ifstream fin;
	string str, strTemp;
	int pos, pos2;

	fin.open (argFilename);
	if (fin.is_open()) {
		// read the first string
		getline (fin, str, '\n');
		pos = (int)str.find_first_of ('?');
		strTemp = str.substr (pos+1, 3);

		if (!strTemp.compare ("xml")) {
			// read another line
			getline (fin, str, '>');
			// read tags recursively
			pos = (int)str.find_first_of ('<');
			pos2 = (int)str.find_first_of (' ', pos);
			if (pos2 < 0) {	// no attributes
				_root._name = str;
			} else {
				_root._name = str.substr (pos+1, (pos2-pos-1));
			}
			readTag (fin, _root);
			return Q_OK;

		} else {	// not an xml
			cerr<<"[QLightXMLIO::readFromFile] error:\n\t"
				<<"Input file ("<<argFilename<<") is not in xml format\n";
		}
		fin.close ();
	} else {
		cerr<<"[QLightXMLIO::readFromFile] error:\n\t"
			<<"Can't read file ("<<argFilename<<")\n";
	}
//	FILE* fin;

//	fin = fopen (argFilename, "rb");

	return Q_OK;
}

int QLightXMLIO::writeToFile (const char* argFilename) const {
	ofstream fout;

	fout.open (argFilename);
	if (fout.is_open ()) {
		// write xml header
		fout<<"<?xml version=\"1.0\"?>\n";
		writeTag (fout, _root);
		fout.close ();
	} else {
		cerr<<"[QLightXMLIO::writeToFile] error:\n\t"
			<<"Can't read file ("<<argFilename<<")\n";
	}

	return Q_OK;
}

int QLightXMLIO::getTagByName (const char* argName, QXMLTAG_LIST& argList) const {
	appendTagToList (_root, argName, argList);

	return Q_OK;
}
