#ifndef		Q_LIGHTXML_H
#define		Q_LIGHTXML_H

#include <iostream>
#include <string>
#include <vector>

using namespace std;

struct QXMLAttribute {
	string _name;
	string _value;
	QXMLAttribute () : _name(""), _value("") {}
	QXMLAttribute (const QXMLAttribute& attr) : _name(attr._name), _value(attr._value) {}
	~QXMLAttribute () {
		_name.clear();
		_value.clear();
	}
};

typedef	vector<QXMLAttribute>	QXMLATTRIBUTES_LIST;
struct QXMLTag;
typedef	vector<QXMLTag>	QXMLTAG_LIST;

struct QXMLTag {
	string _name;		// name of the tag
	string _contents;	// the string between opening and closing tags
	QXMLTAG_LIST _tags;	// list of lower-level tags
	QXMLATTRIBUTES_LIST _attrs;

	QXMLTag () : _name(""), _contents("") {
		_tags.clear ();
		_attrs.clear ();
	}
	QXMLTag (const QXMLTag& tag) : _name(tag._name), _contents(tag._contents) {
		_tags.assign (tag._tags.begin(), tag._tags.end());
		_attrs.assign (tag._attrs.begin(), tag._attrs.end());
	}
	~QXMLTag () {
		_name.clear();
		_contents.clear();
		_tags.clear ();
		_attrs.clear ();
	}
};

/**
*
*	This is light XML reader/writer. 'Light' because it doesn't read attributes
*	and each tag is supposed to be in separate string. If there are attributes they
*	are simply ignored.
*
*/

class QLightXMLIO {
protected:
	QXMLTag _root;	// the top-level tag

	int appendTagToList (const QXMLTag& tag, const char* argName, QXMLTAG_LIST&) const;

public:
	QLightXMLIO ();
	~QLightXMLIO ();

	// Reads tag and all its subtags from the stream
	int readTag (istream& in, QXMLTag& tag);
	// Writes tag and all its subtags into the stream
	int writeTag (ostream& out, const QXMLTag& tag) const;

	// Read xml-tree from file
	int readFromFile (const char* argFilename);
	// Write xml-tree to file
	int writeToFile (const char* argFilename) const;

	// Gets tags with specified name from _root tag and appends them to the list
	int getTagByName (const char* argName, QXMLTAG_LIST&) const;
};

#endif
