#pragma once

#ifndef _QTOOLS_
#define _QTOOLS_

#include <fstream>
#include <iostream>
#include <map>
#include <string>
#include <vector>

using namespace std;

struct QRGB;
class QImage;

typedef unsigned char				__QBYTE;
typedef unsigned short				__QWORD;
typedef unsigned long				__QDWORD;
typedef long						__QLONG;

typedef vector<unsigned int>		__VECTOR_UINT;
typedef vector<int>					__VECTOR_INT;
typedef vector<__VECTOR_INT>		__TABLE_INT;
typedef vector<__QBYTE>				__VECTOR_BYTE;
typedef vector<__VECTOR_BYTE>		__TABLE_BYTE;
typedef vector<double>				__VECTOR_DOUBLE;
typedef vector<__VECTOR_DOUBLE>		__TABLE_DOUBLE;
typedef vector<string>				__VECTOR_STRING;
typedef vector<QRGB>				__VECTOR_RGB;
typedef vector<__VECTOR_RGB>		__TABLE_RGB;
typedef vector<QImage>				__VECTOR_IMAGE;

typedef __VECTOR_INT	QBinaryChromosome;
typedef vector<QBinaryChromosome>	QBinaryPopulation;

typedef map<double, int>			__MAP_DOUBLE_INT;


//typedef valarray<double>			__VALARRAY_DOUBLE;
//typedef valarray<__VALARRAY_DOUBLE>	__VALTABLE_DOUBLE;

#define Q_FAILURE	0
#define Q_OK		1
#define Q_CONVERGENCE	2

#define QRAND_MAX 18446744073709551615;	// 2^64 - 1 as max __int64
#define QM_PI 3.14159265359			// Pi
#define QM_2PI 6.28318530718		// 2Pi
#define QM_1_2PI 0.15915494309	// 1/2Pi
#define QM_PI_2 1.570796326795	// Pi/2
#define QM_EXP 2.718281828459045	// exp
#define QMAX_SIGNED 2147483647        // 2^31 - 1 as max signed integer
#define QMAX_SIGNED_D 2147483647.0    // 2^31 - 1 as floating-point format number
#define QMAX_UNSIGNED 4294967295        // 2^32 - 1 as max integer
#define QMAX_UNSIGNED_D 4294967295.0    // 2^32 - 1 as floating-point format number


static __int64 bitMask[] = {0x01, 0x02, 0x04, 0x08,
														0x010, 0x020, 0x040, 0x080,
														0x0100, 0x0200, 0x0400, 0x0800,
														0x01000, 0x02000, 0x04000, 0x08000,
														0x010000, 0x020000, 0x040000, 0x080000,
														0x0100000, 0x0200000, 0x0400000, 0x0800000,
														0x01000000, 0x02000000, 0x04000000, 0x08000000,
														0x010000000, 0x020000000, 0x040000000, 0x080000000,

														0x0100000000, 0x0200000000, 0x0400000000, 0x0800000000,
														0x01000000000, 0x02000000000, 0x04000000000, 0x08000000000,
														0x010000000000, 0x020000000000, 0x040000000000, 0x080000000000,
														0x0100000000000, 0x0200000000000, 0x0400000000000, 0x0800000000000,
														0x01000000000000, 0x02000000000000, 0x04000000000000, 0x08000000000000,
														0x010000000000000, 0x020000000000000, 0x040000000000000, 0x080000000000000,
														0x0100000000000000, 0x0200000000000000, 0x0400000000000000, 0x0800000000000000,
														0x01000000000000000, 0x02000000000000000, 0x04000000000000000, 0x08000000000000000};

static __int64 splitMask[] = {0x0, 0x1, 0x3, 0x7,
								0xf, 0x1f, 0x3f, 0x7f,
								0xff, 0x1ff, 0x3ff, 0x7ff,
								0xfff, 0x1fff, 0x3fff, 0x7fff,
								0xffff, 0x1ffff, 0x3ffff, 0x7ffff,
								0xfffff, 0x1fffff, 0x3fffff, 0x7fffff,
								0xffffff, 0x1ffffff, 0x3ffffff, 0x7ffffff,
								0xfffffff, 0x1fffffff, 0x3fffffff, 0x7fffffff,
								0xffffffff, 0x1ffffffff, 0x3ffffffff, 0x7ffffffff,
								0xfffffffff, 0x1fffffffff, 0x3fffffffff, 0x7fffffffff,
								0xffffffffff, 0x1ffffffffff, 0x3ffffffffff, 0x7ffffffffff,
								0xfffffffffff, 0x1fffffffffff, 0x3fffffffffff, 0x7fffffffffff,
								0xffffffffffff, 0x1ffffffffffff, 0x3ffffffffffff, 0x7ffffffffffff,
								0xfffffffffffff, 0x1fffffffffffff, 0x3fffffffffffff, 0x7fffffffffffff,
								0xffffffffffffff, 0x1ffffffffffffff, 0x3ffffffffffffff, 0x7ffffffffffffff,
								0xfffffffffffffff, 0x1fffffffffffffff, 0x3fffffffffffffff, 0x7fffffffffffffff,
								0xffffffffffffffff};

//enum QExceptionType {QET_UNKNOWN=0, QET_ILLEGAL_INDEX, QET_ILLEGAL_POINTER, QET_ILLEGAL_OBJECT};

//////////////////////////////////////////////////////////////////////
// Declarations

class QConfigIO;
class QConfifParameter;
class QConfigParameterNode;
class QConfigParameterList;

class QDoubleArray;
class QDoubleArrayNode;
class QDoubleArrayList;

void QInitRand (unsigned);
unsigned int QRand ();
bool QRandBool ();
double QRandN ();		// uniformly distributed on [0, 1] random number
double QRandGauss ();   // random number with N(0,1) distribution

//////////////////////////////////////////////////////////////////////
// Miscellaneous functions

// roulette selection according to the proportions of the entries
// set at the argV array
// Returns index of the selected entry
int roulette (const __VECTOR_INT& argV, int INITIAL_VALUE=1);

// roulette selection according to the proportions of the entries
// set at the argV array
// Returns index of the selected entry
int roulette (const __VECTOR_DOUBLE& argV, int INITIAL_VALUE=1);

// checks whether the given matrix of bytes contains all-zeroes
bool isNull (const __TABLE_BYTE& argM);

const __TABLE_BYTE operator* (const __TABLE_BYTE& argL, const __TABLE_BYTE& argR);

double qavg (__VECTOR_DOUBLE& argV);
double qavg (__VECTOR_INT& argV);
double qsum (__VECTOR_DOUBLE& argV);
int qsum (__VECTOR_INT& argV);
// Calculates mean and variance (not deviation!) of the given array
void qstats (__VECTOR_DOUBLE& argV, double& mean, double& stdvar);

// Calculates mean and variance (not deviation!) of the given array
void qstats (__VECTOR_INT& argV, double& mean, double& stdvar);

// Converts given map (table) to the hystogram
//	argS - source map
//  argT - target map
//  argN - number of categories
double qgethistogram (const __MAP_DOUBLE_INT& argS, __MAP_DOUBLE_INT& argT, const int argN);

int qstrcpy (char*& target, const char* source);
int qstrcpy0 (char* target, char* source);
int char2int (int& target, const char* source);
double char2dbl (double& target, const char* source);
int dbl2char (char*& target, const double source);
int int2char (char* target, const int source);

int trim (string&);

// Converts hex-source in chars to the corresponding decimal value
//	sourceSize - number of bytes in source array (!! can't be more than 4 !!)
int hex2dec (int& target, const char* source, const int sourceSize);

double logsigmoid (const int size, const double *inputs, const double* weights, const double aspect, const double offset);

//int read_ann_data (const char* filename, QDoubleArrayList& ins, QDoubleArrayList& outs);
int read_ann_data (const char* filename, __TABLE_DOUBLE& ins, __TABLE_DOUBLE& outs);

// int readBMP (const char *argFilename, __TABLE_RGB& argImage)
//	Reads uncompressed BMP-file and fills argImage container with corresponding RGB-values
//	Returns non-zero on success
//int readBMP (const char *argFilename, __TABLE_RGB& argImage);

// int readBMP (const char *argFilename, QRGB** argImage, int& argWidth, int& argHeight)
//	Reads uncompressed BMP-file and fills argImage container with corresponding RGB-values
//	Returns non-zero on success
int readBMP (const char *argFilename, QRGB**& argImage, int& argWidth, int& argHeight);


// int writeBMP (const char *argFilename, const QRGB** argImage, const int argWidth, const int argHeight)
//	Writes uncompressed 24-bit BMP-file with specified filename
//	Returns non-zero on success
int writeBMP (const char *argFilename, QRGB** const argImage, const int argWidth, const int argHeight);


// int autolevels (QRGB** const argImage, const int argWidth, const int argHeight)
//	Automatically equalizes image histogram using algorithm from (Gonsalez, 2002;
//	see scans for more details)
//	Returns non-zero on success
int qhisteq (QRGB** argImage, const int argWidth, const int argHeight);


// Apply Sobel's transormation to detect edges
//	The source image is supposed to be a grey-level 24-bit image
//	Result is stored in the same image!
int edgeSobel (QImage& argImage, const double argT);

// Calculates Peak Signal-Noise Ratio for given 2D arrays
double calculatePSNR (const __TABLE_DOUBLE& argRef, const __TABLE_DOUBLE& argT);

// Evaluates image quality using approach from ANN training
double evaluateImage (const QImage * argImage);

// Evaluates image quality using modified approach from ANN training (with respect to mean intensity of edgels)
double evaluateImageMod (const QImage * argImage);

// Evaluates image quality according to the approach by C. Munteanu and A. Rosa
double evaluateImageMunteanuRosa (const QImage * argImage);

//////////////////////////////////////////////////////////////////////
// Miscellaneous classes and structures

struct QRGB {
	__QBYTE b;
	__QBYTE g;
	__QBYTE r;
};

struct QRGBa {
	__QBYTE r;
	__QBYTE g;
	__QBYTE b;
	__QBYTE alpha;
};

// BMP-file header
// For proper use of this structure either use its components separately or set alignment to 2 bytes
//	(single word), because sizeof(QBMPHeader) returns 16 (!!)
struct QBMPHeader {
	__QWORD signature;	// file signature (should be 0x4d42 = "BM")
	__QDWORD size;		// file-size
	__QWORD res1;		// reserved
	__QWORD res2;		// reserved
	__QDWORD offset;	// offset of the image info from the beginning of the file
};	// 14 bytes

// BMP informational header
struct QBMPInfoHeader {
	__QDWORD size;	// size of header
	__QLONG width;	// image width (in pixels)
	__QLONG height;	// image height (in pixels)
	__QWORD planes;	// number of planes (deprecated, for low-color images only)
	__QWORD bitsPerPixel;
	__QDWORD compression;	// compression type (0 if not compressed)
	__QDWORD sizeImage;		// image size (in bytes)
	__QLONG pixelsPerMeterX;	// horizontal resolution (pixels per meter)
	__QLONG pixelsPerMeterY;	// vertical resolution (pixels per meter)
	__QDWORD colorsUsed;	// number of used colors (0 if max possible for given color depth)
	__QDWORD colorsMain;	// number of main colors
};	// 40 bytes

struct QBMPColorTable {
	QRGBa colors[256];
};

class QImage {
protected:
	QRGB** _image;
	int _width, _height;
	__QBYTE _maxR, _maxG, _maxB;	// max values of color components
	__QBYTE _minR, _minG, _minB;	// min values of color components
	__QBYTE _maxBrightness;
	double _avR, _avG, _avB;
	double _avBrightness, _devBrightness;

public:

	bool ignoreOutliers;

	QImage ();
	QImage (const QImage& argImage);
	virtual ~QImage ();

	QRGB* operator[] (const int index);
	QRGB* operator[] (const int index) const;
	QImage& operator= (const QImage&);
	bool operator== (const QImage&);

	int readFromFile (const char* argFilename);
	int writeToFile (const char* argFilename);
	int writeToFile (const char* argFilename) const;

	int getHeight () const {return _height;}
	int getWidth () const {return _width;}
	double getAverageR () const {return _avR;}
	double getAverageG () const {return _avG;}
	double getAverageB () const {return _avB;}
	double getMaxBrightness () const {return _maxBrightness;}
	double getMeanBrightness () const {return _avBrightness;}
	double getDeviationBrightness () const {return _devBrightness;}
	QRGB* getPixel (const int i, const int j) const {return &_image[i][j];}

	// Copies the red plane into the argImg
	int getPlaneR (QImage* argImg);
	// Copies the green plane into the argImg
	int getPlaneG (QImage* argImg);
	// Copies the blue plane into the argImg
	int getPlaneB (QImage* argImg);

	// Sets the size of the image
	//
	// !!!!!!!!!!! Caution !!!!!!!!!!!
	// !! All image data is deleted !!
	// !!!!!!!!!!! Caution !!!!!!!!!!!
	void setDimensions (const int argW, const int argH);

	void eraseImage ();

	// Converts RGB-image to the grayscale color space using the following formula (see FAQ about Color and Gamma, C-9):
	// Y = 0.212671 * R + 0.715160 * G + 0.072169 * B;
	int toGrayscale ();

	// Converts RGB-image to the grayscale color space using simple averaging formula:
	// Y = 0.333333 * (R + G + B);
	int toGrayscaleAvg ();

	// Converts RGB-image to the grayscale color space using the NTCS-formula (see FAQ about Color and Gamma, C-9):
	// Y = 0.299 * R + 0.587 * G + 0.114 * B;
	int toGrayscaleNTSC ();

	// Converts greyscale image into RGB-color space using given color map
	int toColor (const QImage& argImage);

	// Edge detection using Sobel's transformation
	int edgeSobel ();

	// Calculates pixel with luma value larger than value of the argument
	//	Returns number of calculated pixels
	//	The image is supposed to be a grey-scale(!)
	int calculatePixels (const double argLuma);

	// double calculateSumBrightness (double argL)
	// The image is supposed to be a *grey-scale* !!!
	double calculateSumBrightness (double argL);

	// Recalculates _avR, _avG and _avB
	//	_avLuma is resalculated also and is set equal to _avB
	int recalculateStatistics ();

	// double calculateEntropy ()
	// Calculates entropy of brightness. The image is supposed to be
	//	a *grey-scale* !!!
	double calculateEntropy ();

	// Calculates monochrome (!) histogram of the image and makes appropriate BMP-file
	int calculateHistogram (const char* argFilename);

	// Calculates greyscale normalized histogram of the image and writes result in the
	//  passed array
	int calculateHistogram (__VECTOR_DOUBLE& argRes);

	// Calculates normalized histogram of the image for RED channel and writes result in the
	//  passed array
	int calculateHistogramR (__VECTOR_DOUBLE& argRes);

	// Calculates normalized histogram of the image for GREEN channel and writes result in the
	//  passed array
	int calculateHistogramG (__VECTOR_DOUBLE& argRes);

	// Calculates normalized histogram of the image for BLUE channel and writes result in the
	//  passed array
	int calculateHistogramB (__VECTOR_DOUBLE& argRes);

	// changes lightness using multiplication by argAspect value (positive only)
	int changeLightnessMult (const double argAspect);

	// changes lightness by addition of argAspect value (can be both negative and positive)
	int changeLightnessAdd (const int argAspect);

	// Enhances image using analogue of Matlab's imadjust method. I.e.
	// 1% of data is saturated at low and high image's intencities while
	// the other part of data is mapped over the available intencities range
	int matlabImadjust ();

	// Make histogram equalization
	int histogramEqualization ();

	// fills the entire image with white color
	int fillWhite ();

	// int calculateNeighborhood (const int argSize, __TABLE_DOUBLE &argM, __TABLE_DOUBLE &argD)
	//	Calculates neighbourhood statistics (mean and deviation) using exact formulae
	//	Returns time (in clocks, divide on CLOCK_PER_SEC) required to perform computations
	//	!!! Normalized values [0..1] are written in the argument arrays !!!
	int calculateNeighborhood (const int argSize, __TABLE_DOUBLE &argM, __TABLE_DOUBLE &argD) const;

	// int calculateNeighborhoodApprx (const int argSize, __TABLE_DOUBLE &argM, __TABLE_DOUBLE &argD)
	//	Calculates neighbourhood statistics using (mean and deviation) approximate formulae
	//	Returns time (in clocks, divide on CLOCK_PER_SEC) required to perform computations
	//	!!! Normalized values [0..1] are written in the argument arrays !!!
	int calculateNeighborhoodApprx (const int argSize, __TABLE_DOUBLE &argM, __TABLE_DOUBLE &argD) const;

	// int calculateNeighborhoodApprxOnlyLocal (const int argSize, __TABLE_DOUBLE &argM, __TABLE_DOUBLE &argD)
	//	Calculates neighbourhood statistics using (mean and deviation) approximate formulae without
	//  respect to global mean intensity!
	//	Returns time (in clocks, divide on CLOCK_PER_SEC) required to perform computations
	//	!!! Normalized values [0..1] are written in the argument arrays !!!
	int calculateNeighborhoodApprxOnlyLocal (const int argSize, __TABLE_DOUBLE &argM, __TABLE_DOUBLE &argD) const;

	bool isMonochrome ();
};

class QFibonacciSequence {
protected:
	int a0, a1;
	int current;
	int position;
public:
	QFibonacciSequence () : a0(1), a1(1), current(1), position(1) {}
	virtual ~QFibonacciSequence () {}

	int operator++ () {
		if (position > 1) {
			position++;
			a0 = a1;
			a1 = current;
			current += a0;
			return current;
		} else {
			position++;
			return 1;
		}
	}
	int operator-- () {
		if (position > 2) {
			position--;
			current = a1;
			a1 = a0;
			a0 = current - a1;
			return current;
		} else {
			a0 = a1 = current = position = 1;
			return 1;
		}
	}
	int getCurrent () const {return current;}
	int getPosition () const {return position;}
	void reset () {
		a0 = a1 = current = position = 1;
	}
};

// One-dimensional array of doubles
class QDoubleArray {
protected:
	double *Items;
	int Size;
public:
	QDoubleArray ();
	QDoubleArray (const int size);
	QDoubleArray (const int size, const double*);
	QDoubleArray (const QDoubleArray&);
	virtual ~QDoubleArray ();

	virtual QDoubleArray& operator= (const QDoubleArray&);
	virtual double& operator[] (const int) const;

	virtual int SetSize (const int size);
	virtual int GetSize () {return Size;}
	virtual int Reset ();
	bool IsEmpty () const {return (Size != 0)? false : true;}
};

class QDoubleArrayNode : public QDoubleArray {
public:
	int Index;
	QDoubleArrayNode *Next;

	QDoubleArrayNode ();
	QDoubleArrayNode (const int size, const double* items, const int index);
	virtual ~QDoubleArrayNode ();

	virtual int PrintInfo ();
};

// List of one-dimensional arrays of doubles.
// Two-dimensional array of doubles to say exactly.
class QDoubleArrayList {
protected:
	QDoubleArrayNode *First, *Last;
public:
	int Count;

	QDoubleArrayList ();
	virtual ~QDoubleArrayList ();

	virtual QDoubleArray& operator[] (const int);

	virtual int Add (const int size);
	virtual int Add (const int size, const double*);
	virtual int Add (const QDoubleArrayNode&);
	virtual int Reset ();

	virtual int PrintInfo ();
};

//////////////////////////////////////////////////////////////////////
// Classes for config files IO-operations
// Not complete yet
//
enum QConfigParameterType {CPT_UNKNOWN, CPT_TEXT, CPT_NUMBER, CPT_DATETIME};

class QConfigParameterValue {
	char *cValue;
	double nValue;
public:
	QConfigParameterValue ();
	virtual ~QConfigParameterValue ();
};

class QConfigParameter {
	friend class QConfigParameterList;
	friend class QConfigIO;
protected:
	char *Name,
			 *cValue;
	double nValue;
public:
	QConfigParameterType Type;

	QConfigParameter ();
	QConfigParameter (const char*);
	QConfigParameter (const char* name, const QConfigParameterType cpt, const char* value);
	QConfigParameter (const QConfigParameter&);
	virtual ~QConfigParameter ();

	virtual QConfigParameter& operator= (const QConfigParameter&);
	virtual QConfigParameter& operator= (const char*);
	virtual QConfigParameter& operator= (const double);
	friend ostream& operator<< (ostream& stream, const QConfigParameter& argParam);

	virtual char* GetValueC ();
	virtual double GetValueN ();
	virtual char* GetName () {return Name;}
	virtual int SetName (const char*);
	virtual int SetValue (const char*);
	virtual int SetValue (const double);
	virtual int SetType (const QConfigParameterType);
};

typedef	vector<QConfigParameter>	__QCONFIG_PARAMETER_LIST;

class QConfigIO {
protected:
	virtual int ReadName (ifstream& fin, char*& target);
	virtual int ReadValue (ifstream& fin, char*& target);
	virtual int ReadParameter (ifstream& fin, char* name, char* value);
	virtual int NextLine (ifstream& fin);
public:
	__QCONFIG_PARAMETER_LIST Params;

	QConfigIO ();
	virtual ~QConfigIO ();

	virtual int ReadFromFile (const char* filename);
	virtual int ReadFromStream (const istream&);
	virtual int WriteToFile (const char* filename);
	virtual int WriteToStream (const ostream&);
};

#endif
